/*************************************************************************
  2bl.c

  Copyright (c) Chumby Industries 2007                bunnie@chumby.com
  Based on code by Freescale Suzhou Design Center
  Dual MPL/GPL2 license

  "The contents of this file are subject to the Mozilla Public License
  Version 1.1 (the "License"); you may not use this file except in
  compliance with the License. You may obtain a copy of the License at
  http://www.mozilla.org/MPL/

  Software distributed under the License is distributed on an "AS IS"
  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
  License for the specific language governing rights and limitations
  under the License.

  The Original Code is from the Freescale MX21 ADS SDK.

  The Initial Developer of the Original Code is the Freescale Suzhou 
  Design Center. Portions created by Chumby Industries are Copyright (C)
  2007 Chumby Industries. All Rights Reserved.

  Contributor(s): bunnie@chumby.com
  Modifications:  ken@chumby.com - 11/14/08 - removed support for clearing MSP
  
  Alternatively, the contents of this file may be used under the terms
  of the BSD license (the  "[BSD] License"), in which case the
  provisions of [BSD] License are applicable instead of those
  above.  If you wish to allow use of your version of this file only
  under the terms of the [BSD] License and not to allow others to use
  your version of this file under the MPL, indicate your decision by
  deleting  the provisions above and replace  them with the notice and
  other provisions required by the [BSD] License.  If you do not delete
  the provisions above, a recipient may use your version of this file
  under either the MPL or the [BSD] License."

  "Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
      * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
      * Neither the name of the Chumby Industries nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.
  
  THIS SOFTWARE IS PROVIDED BY Chumby Industries ``AS IS'' AND ANY
  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL Chumby Industries BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."

*************************************************************************/

// the order of execution is:
// boot.S -> bl.c -> 2bl.c -> linux kernel

/*
  Note that this code assumes that the *code region* at least fits
  entirely into a good page of memory (guaranteed to be good for the first
  16384 bytes). If the code size exceeds this amount, you run the risk of
  having it roll over into a "bad page" and the boot proces being halted
  unceremoniously by a bad block. 

  In other words, this should compile to a size of less than 14336 bytes.
  Will the world end if it doesn't? No. However, you run roughly a
  0.024% chance of having a device not work if it spills over into the
  next bad block (it might just be cheaper to rework that ROM chip
  with one that doesn't have a bad block there!).

  Note that this code does have mechanisms to account for bad blocks
  in the display bitmaps embedded in the bootloader. 
*/

//<<<<<Include
#ifndef _STANDALONE_
#define _STANDALONE_
#endif 
#include "mx2.h"

#include "mx2_nand.h"
#include "uart.h"
#include "usb.h"
#include "touchpanel.h"

#include "common.h"

#include "../linux/include/asm-arm/mach-types.h"

#define NULL ((void *) 0)
//>>>>>Include

#define USE12M_XTAL 0
#define USE16M_XTAL 1

//<<<<<< Private Macro
#define	CMD_LINE_LEN		80
#define	DELAY_LOOP_COUNT 	0x20000    		///< When Bootloader Wait Input,System Delay. 

/// Kernel Start Address In SDRAM.
//#define KERNEL_START_ADDRESS 0xc0008000	
#define KERNEL_START_ADDRESS 0xc0500000	 // move here for zImage booting

/// Kernel Max Size (Unit is Page) -- up to 4 megabytes
#define KERNEL_SIZE	     (0x400000/NAND_FLASH_PAGE_MAIN_SIZE)

/// Kernel Partition start page of NAND Flash 
//#define NAND_KERNEL_START    (0x100000/NAND_FLASH_PAGE_MAIN_SIZE)
#define NAND_KERNEL1_START    (0xD80000/NAND_FLASH_PAGE_MAIN_SIZE)
#define NAND_KERNEL2_START    (0x300000/NAND_FLASH_PAGE_MAIN_SIZE)

#define NAND_TEST_CODE_START       (0x44000/NAND_FLASH_PAGE_MAIN_SIZE)
#define NAND_TEST_CODE_LENGTH      (0x3C000/NAND_FLASH_PAGE_MAIN_SIZE)
#define TEST_START_ADDRESS         0xC1F80000

// 0x1000 extra offset due to difference in SDRAM copy pt. vs FLASH mapping - 2BL only
#define B4_1_POINTER  0xC0010800   // B4_1_POINTER is special, it must always be the first image in memory, as it is the loading offset for all other images as well
#define B5_1_POINTER  0xC0036000
#define B5_2_POINTER  0xC005B800
#define B4_2_POINTER  0xC0081000
#define B5_3_POINTER  0xC00A6800
#define B5_3_SIZE     0x25800
#define B5_1_SIZE     0x25800    // size in bytes of note image
// kernel uses this next location, don't move it unless you move it in imxfb.c
#define B4_1_BACK_PTR 0xC3000000 // make a copy way out in memory so kernel can be more efficiently located
#define B5_1_BACK_PTR 0xC3200000
#define B4_1_SIZE     0x25800
#define FRAMEBUF_PTR  0xC3400000
#define B5_2_SIZE     0x25800
#define B5_2_BACK_PTR 0xC3600000
#define B4_2_BACK_PTR 0xC3800000
#define B4_2_SIZE     0x25800
#define B5_3_BACK_PTR 0xC3A00000
#define KERNEL_LOGO_PTR 0xC2500000

//#define MSP_OFFSET  0x40000   /* offset in FLASH ROM to find MSP semaphore */
#define MSP_OFFSET 0x3FF0000 // 0x4000000 - 0x10000
#define MSP_LENGTH 0x10000
#define MSP_BUFFER 0xC1500000

#define DRAM_BASE  0xC0000000

/*
  New MSP schema:
  
  look for MSP in the top of FLASH, top 4 sectors (64 kb)
  ignore bad block bits when reading and writing
  read the 64k of data and sum the number of 1's and 0's in those regions
  if there are more 1's than zeros, then assume "erased"
  otherwise, assume "not erased"
  
  do not do bad block translation on the MSP sectors
*/

#define TIMEOUT 700000 // calibrate to about four seconds

#define NAND_PAGE_SIZE  512   // in bytes
#define NAND_BLOCK_SIZE 16384 // in bytes
#define MAX_BLOCKS      4096  // for a 512 Mbit ROM
#define BB_TBL_LOC      0xC1200000
#define BB_TBL_PTR      ((unsigned int *) BB_TBL_LOC)

#define DEBUG_MSP 0
#define DEBUG 0
#define DEBUG_DRIVE 0
#define DEBUG_BLOCKKILLER 0

extern void setup_tags(void *parameters, int doUpdateBoot, int hwVersion);

//<<<<<Body

unsigned int xlat( unsigned int page ) {
  return((BB_TBL_PTR[(page * NAND_PAGE_SIZE) / (NAND_BLOCK_SIZE)] + page * NAND_PAGE_SIZE) / NAND_PAGE_SIZE );
}

void printhex(char *buf, unsigned int num) {
  int i;
  for( i = 0; i < 8; i++ ) {
    buf[i] = ((num & 0xF0000000) >> 28) & 0xF;
    buf[i] = (buf[i] > 9) ? (buf[i] + 'A' - 10) : (buf[i] + '0');
    num <<= 4;
  }
  buf[i] = '\0';
}

void printhexchar(char *buf, unsigned char num) {
  int i;
  for( i = 0; i < 2; i++ ) {
    buf[i] = ((num & 0xF0) >> 4) & 0xF;
    buf[i] = (buf[i] > 9) ? (buf[i] + 'A' - 10) : (buf[i] + '0');
    num <<= 4;
  }
  buf[i] = '\0';
}

void BBscan() {
  int i, j;
  unsigned char spare[16];
  unsigned int blockOffset = 0;
  char numBuf[20];
  int foundBad = 0;
  
  for( i = 0; i < MAX_BLOCKS; i++ ) {
    NAND_Read_OOB(0, (i * NAND_BLOCK_SIZE) / NAND_PAGE_SIZE, spare, 16);
    if( spare[5] != 0xFF ) {
      // found a bad block
      blockOffset += NAND_BLOCK_SIZE;
      EUARTputString( "Block 0x" );
      printhex(numBuf, i);
      EUARTputString(numBuf);
      EUARTputString( " is bad.\n" );
      foundBad = 1;
      // compensate for non-causal bad blocks: e.g., consecutive bad blocks
      // or, the blocks you didn't know about between you and your "real" good block
      // back when you calculated your offset
      for( j = 0; j < blockOffset / NAND_BLOCK_SIZE; j++ ) {
	BB_TBL_PTR[i-j] += NAND_BLOCK_SIZE;
      }
    }
    // keep this clause because [i]th element still isn't initialized in the above math
    BB_TBL_PTR[i] = blockOffset;
    
  }
  if( foundBad ) {
    EUARTputString( "80 bad blocks or less are normal for this NAND FLASH.\n" );
  }
}

///Init Mx2 Hardware. 
void init()
{
  int dummy,reg;
  
  // burst-flash initialization
  _reg_SYS_FMCR=0xFFFFFFC1; // changed bunnie
  
  
  // Initialize all peripheral in AIPI1 PSR[1:0] => 10=32bit, 01=16bit, 00=8bit
  _reg_AIPI1_PSR0 = 0x00040304;
  _reg_AIPI1_PSR1 = 0xFFFBFCFB;
  
  // Initialize all peripheral in AIPI1 PSR[1:0] => 10=32bit, 01=16bit, 00=8bit
  _reg_AIPI2_PSR0 = 0x0;
  _reg_AIPI2_PSR1 = 0xFFFFFFFF;
  
  //write to the FMCR [31:24] (CLKMODE[1:0]) in order to get the write enable signal active
  _reg_SYS_FMCR |= 0xAA000000;
  
}

/// Copy Kernel From NAND Flash to SDRAM
// when doUpdateBoot is 1, go to K2
// otherwise, go to K1
void CopyKernelToMem(int doUpdateBoot)
{
  char 	*pDest = (char*)KERNEL_START_ADDRESS;
  char 	spare[NAND_FLASH_PAGE_SPARE_SIZE];
  int 	i;
  int kernelStartPage = NAND_KERNEL2_START;
  char numBuf[20];
  
  if( doUpdateBoot )
    kernelStartPage = NAND_KERNEL2_START;
  else
    kernelStartPage = NAND_KERNEL1_START;
  
  for(i = kernelStartPage; i < kernelStartPage + KERNEL_SIZE; i++)
    {
#ifdef NAND_ECC
      if(i%32==0) //One block
	{
	  if(NAND_Read_OOB(0,i,spare,16))
	    {
	      EUARTputString("Read OOB Data Error\n");
	      return;
	    }
	  if(spare[5]!=0xFF)
	    {
	      //Bad Block
	      EUARTputString("A Bad Block\n");
	      i+=31;
	      continue;
	    }
	}
#endif
#if 0
      if( (i % 32) == 0 ) {
	EUARTputString( "getting " );
	printhex(numBuf, xlat(i) * NAND_PAGE_SIZE);
	EUARTputString(numBuf);
	EUARTputString( " to " );
	printhex(numBuf, (unsigned long) pDest);
	EUARTputString(numBuf);
	EUARTputString( "\n" );
      }
#endif
      if(NAND_Read_OnePage(xlat(i), pDest, spare)) {
	EUARTputString("Read Data Error\n");
	return;
      }
      
      if(i%256==0)
	EUARTputString(".");
      
#ifdef NAND_ECC
      // ECC handle after read
      mx2_read_ecc(pDest, spare, i);
#endif
      
      pDest+=NAND_FLASH_PAGE_MAIN_SIZE;	
    }
}

void touchpanel_init() {
#if DEBUG
  EUARTputString("touchpanel_init: gpio_init()\n");
#endif
  gpio_init();
#if DEBUG
  EUARTputString("touchpanel_init: spi_init()\n");
#endif
  spi_init();
#if DEBUG
  EUARTputString("touchpanel_init: tsc2100_touchscreen_enable()\n");
#endif
  tsc2100_touchscreen_enable();
  
}

void LCDC_Disable(void);
void LCDC_Enable(void);
void LCDC_Init(int versionCode, unsigned long *lcdBufLoc);
extern int  SysInit(void);

void printNum(unsigned long num) {
  char buf[2];
  int i;
  int digit;
  
  buf[1] = '\0';
  
  for( i = 0; i < 8; i ++ ) {
    digit = ((num & 0xF0000000) >> 28);
    if( digit > 9 )
      buf[0] = 'A' + digit - 10;
    else
      buf[0] = '0' + digit;
    EUARTputString(buf);
    num <<= 4;
  }
}

#define DEBUG_LOAD_PROBLEM 0
void blankScreen(int versionCode) {
#if DEBUG_LOAD_PROBLEM
  unsigned char *buffer = (unsigned char *) 0xC0010000;
#endif
  /* Disable the LCDC */
  
  //  EUARTputString("SysInit\n" );
  SysInit(); // for LCD
#if DEBUG
  EUARTputString(" sysinit\n");
#endif
  
  //  EUARTputString("EnableICache\n" );
  EnableICache();
#if DEBUG
  EUARTputString(" cache\n");
#endif
  
  //  EUARTputString("LCDC_Disable\n" );
  LCDC_Disable();
#if DEBUG
  EUARTputString(" disable\n");
#endif
  
  int i; 
  for( i = 0; i < B4_1_SIZE; i++ ) {
#if DEBUG
    if( (i % 1024) == 0)
      EUARTputString(".");
#endif
    //    *((unsigned long *) FRAMEBUF_PTR + i) = (unsigned long) 0xFFFFFFFF;
    *((unsigned long *) FRAMEBUF_PTR + i) = *((unsigned long *) B4_1_BACK_PTR + i);
  }
#if DEBUG
  EUARTputString(" blank\n");
#endif
  
  /* Set up the LCDC for the attached LCD panel */
  //  EUARTputString("LDCD_Init\n" );
  LCDC_Init(versionCode, (unsigned long *) FRAMEBUF_PTR);
  //LCDC_Init(B4_1_BACK_PTR);
#if DEBUG
  EUARTputString(" init\n");
#endif
  
  /* Enable the LCDC */
  //  EUARTputString("LCDC_Enable\n" );
  LCDC_Enable();
#if DEBUG
  EUARTputString(" enable\n");
#endif
  
  /* Main screen buffer points to image data */
  //  *(p_uint32_t)(LCDC_LSSAR) = (uint32_t)flamingos;
  //  EUARTputString("setting B4_1_POINTER\n" );
  
#if DEBUG_LOAD_PROBLEM
  EUARTputString("start buffer: 0x");
  printNum((unsigned long) buffer);
  EUARTputString("\n");
  while( *buffer != 0xDF )
    buffer++;
  
  buffer -= 0xB94;
  
  EUARTputString("found buffer: 0x");
  printNum((unsigned long) buffer);
  EUARTputString("\n");
  _reg_LCDC_LSSAR = (unsigned long) buffer;
  EUARTputString("using: 0x");
  printNum((unsigned long) B4_1_POINTER);
  EUARTputString("\n");
  EUARTputString("0xC0001000: 0x");
  printNum(*(unsigned long *) 0xC0001000);
  EUARTputString(" ");
  printNum(*(unsigned long *) 0xC0001004);
  EUARTputString(" ");
  printNum(*(unsigned long *) 0xC0001008);
  EUARTputString(" ");
  printNum(*(unsigned long *) 0xC000100C);
  EUARTputString("\n");
#endif
  
  _reg_LCDC_LSSAR = (unsigned long) FRAMEBUF_PTR;
}

// this fires off the test program
void copyTestCode() {
  char 	*pDest = (char*)TEST_START_ADDRESS;
  int 	i;
  
  for(i = NAND_TEST_CODE_START; i < NAND_TEST_CODE_START + NAND_TEST_CODE_LENGTH; i++)
    {
      if(NAND_Read_OnePage(xlat(i), pDest, NULL))
	{
	  EUARTputString("Read Data Error\n");
	  return;
	}
      
      if(i%256==0)
	EUARTputString(".");
      
      pDest+=NAND_FLASH_PAGE_MAIN_SIZE;	
    }
}

/***
    return 1 if test pin is asserted, otherwise, return 0
    
    this routine checks if one of the GPIOs is strapped and if so
    we fall into test mode!
****/
int testPinAsserted() {
  // need to write this code!!
  
  return 0;
}

/**
   return 1 if touchpanel is being touched, otherwise, return 0
**/
int touchpanel_touched() {
  
  return tsc2100_pendown();
}

void displayTouchMsg() {
  int i;
  
  for( i = 0; i < (B5_1_SIZE * 4); i++ ) {
    *((unsigned char *) (FRAMEBUF_PTR + i)) = *(unsigned char *)(B5_1_BACK_PTR + i);
  }
  //  for( i = 0; i < B5_1_SIZE; i++ ) {
  //    *(unsigned char *)(FRAMEBUF_PTR + i + 320 * 2 * 120) = *(unsigned char *)(B5_1_BACK_PTR + i);
  //  }
}

void displayBypassMsg() {
  int i;
  
  for( i = 0; i < (B5_2_SIZE * 4); i++ ) {
    *((unsigned char *) (FRAMEBUF_PTR + i)) = *(unsigned char *)(B5_2_BACK_PTR + i);
  }
}

// this procedure copies display images to a backing buffer that is high in memory
// beyond the kernel image, so we don't lose a constant 256k due to early storage
// of these display images
void setupBuffers() {
  int i, x, y;
  
#define LOGO_NAND_OFFSET   0x10000
#define LOGO_NAND_END_OFFSET    0x0100000  // cover the whole region where logos could be, correlated to partition size in buildGoldImage.pl
  unsigned char* buff = (unsigned char *) B4_1_POINTER;
  
  // recopy the logo in case we have an offset due to a bad block!
  // yes, this is inefficient--to fix this, optimize the bl.c code to
  // not copy the logo in the first place...but i think it only costs ~200ms or so on boot
  for(i = LOGO_NAND_OFFSET / NAND_PAGE_SIZE; i < LOGO_NAND_END_OFFSET / NAND_PAGE_SIZE; i++) {
    NAND_Read_OnePage( xlat(i), buff, (char*)0);
    buff += NAND_PAGE_SIZE;
  }
  y = 0; x = 0;
  for( i = 0; i < B4_1_SIZE / 2; i++ ) {
    ((unsigned short *) B4_1_BACK_PTR)[x + 640 * y] = ((unsigned short *) B4_1_POINTER)[i];
    ((unsigned short *) B4_1_BACK_PTR)[x + 320 + 640 * y] = ((unsigned short *) B4_1_POINTER)[i];
    x++;
    if( (i % 320) == 319 ) {
      x = 0;
      y++;
    }
  }
  x = 0;
  for( i = 0; i < B4_1_SIZE / 2; i++ ) {
    ((unsigned short *) B4_1_BACK_PTR)[x + 640 * y] = ((unsigned short *) B4_1_POINTER)[i];
    ((unsigned short *) B4_1_BACK_PTR)[x + 320 + 640 * y] = ((unsigned short *) B4_1_POINTER)[i];
    x++;
    if( (i % 320) == 319 ) {
      x = 0;
      y++;
    }
  }

  y = 0; x = 0;
  for( i = 0; i < B5_1_SIZE / 2; i++ ) {
    ((unsigned short *) B5_1_BACK_PTR)[x + 640 * y] = ((unsigned short *) B5_1_POINTER)[i];
    ((unsigned short *) B5_1_BACK_PTR)[x + 320 + 640 * y] = ((unsigned short *) B5_1_POINTER)[i];
    x++;
    if( (i % 320) == 319 ) {
      x = 0;
      y++;
    }
  }
  x = 0;
  for( i = 0; i < B5_1_SIZE / 2; i++ ) {
    ((unsigned short *) B5_1_BACK_PTR)[x + 640 * y] = ((unsigned short *) B5_1_POINTER)[i];
    ((unsigned short *) B5_1_BACK_PTR)[x + 320 + 640 * y] = ((unsigned short *) B5_1_POINTER)[i];
    x++;
    if( (i % 320) == 319 ) {
      x = 0;
      y++;
    }
  }
  
  y = 0; x = 0;
  for( i = 0; i < B5_2_SIZE / 2; i++ ) {
    ((unsigned short *) B5_2_BACK_PTR)[x + 640 * y] = ((unsigned short *) B5_2_POINTER)[i];
    ((unsigned short *) B5_2_BACK_PTR)[x + 320 + 640 * y] = ((unsigned short *) B5_2_POINTER)[i];
    x++;
    if( (i % 320) == 319 ) {
      x = 0;
      y++;
    }
  }
  x = 0;
  for( i = 0; i < B5_2_SIZE / 2; i++ ) {
    ((unsigned short *) B5_2_BACK_PTR)[x + 640 * y] = ((unsigned short *) B5_2_POINTER)[i];
    ((unsigned short *) B5_2_BACK_PTR)[x + 320 + 640 * y] = ((unsigned short *) B5_2_POINTER)[i];
    x++;
    if( (i % 320) == 319 ) {
      x = 0;
      y++;
    }
  }

  y = 0; x = 0;
  for( i = 0; i < B4_2_SIZE / 2; i++ ) {
    ((unsigned short *) B4_2_BACK_PTR)[x + 640 * y] = ((unsigned short *) B4_2_POINTER)[i];
    ((unsigned short *) B4_2_BACK_PTR)[x + 320 + 640 * y] = ((unsigned short *) B4_2_POINTER)[i];
    x++;
    if( (i % 320) == 319 ) {
      x = 0;
      y++;
    }
  }
  x = 0;
  for( i = 0; i < B4_2_SIZE / 2; i++ ) {
    ((unsigned short *) B4_2_BACK_PTR)[x + 640 * y] = ((unsigned short *) B4_2_POINTER)[i];
    ((unsigned short *) B4_2_BACK_PTR)[x + 320 + 640 * y] = ((unsigned short *) B4_2_POINTER)[i];
    x++;
    if( (i % 320) == 319 ) {
      x = 0;
      y++;
    }
  }

  y = 0; x = 0;
  for( i = 0; i < B5_3_SIZE / 2; i++ ) {
    ((unsigned short *) B5_3_BACK_PTR)[x + 640 * y] = ((unsigned short *) B5_3_POINTER)[i];
    ((unsigned short *) B5_3_BACK_PTR)[x + 320 + 640 * y] = ((unsigned short *) B5_3_POINTER)[i];
    x++;
    if( (i % 320) == 319 ) {
      x = 0;
      y++;
    }
  }
  x = 0;
  for( i = 0; i < B5_3_SIZE / 2; i++ ) {
    ((unsigned short *) B5_3_BACK_PTR)[x + 640 * y] = ((unsigned short *) B5_3_POINTER)[i];
    ((unsigned short *) B5_3_BACK_PTR)[x + 320 + 640 * y] = ((unsigned short *) B5_3_POINTER)[i];
    x++;
    if( (i % 320) == 319 ) {
      x = 0;
      y++;
    }
  }
}

void mx21_module_init()
{
  //comment ### Master Priority Register for Slave Port 3
  // Keep LCDC as the highest priority
  _reg_SYS_PCSR = 0;
  _reg_MAX_SLV_MPR(3) = 0x00123056;
  _reg_MAX_SLV_SGPCR(3) = 0;
  
  //comment # enable user mode CSI access
  //setmem 0x10027818 0x6000e 32
  _reg_SYS_GPCR = 0x6000e;
}

int checkMSPsetting() {
  int doUpdateBoot = 1;
  unsigned char *mspBuf = (unsigned char *) MSP_BUFFER;
  int i = 0;
  int numOnes = 0;
  int numZeros = 0;
  char numbuf[20];
  
  /////////////
  ///////// NOTE NOTE NOTE: MSP DOES NOT OPERATE ON BAD BLOCK CORRECTED ADDRESSES
  ///////// it relies instead on multi-block averaging for reliable operation
  /////////////
  for( i = 0; i < MSP_LENGTH; i += NAND_FLASH_PAGE_MAIN_SIZE ) {
    NAND_Read_OnePage((MSP_OFFSET + i) / NAND_FLASH_PAGE_MAIN_SIZE, (unsigned char *)mspBuf + i, NULL);
  }
  for( i = 0; i < MSP_LENGTH; i++ ) {
#if DEBUG_MSP
    if( (i % 16) == 0 ) {
      EUARTputString("\n");
      printhex( numbuf, i );
      EUARTputString(numbuf); EUARTputString(": ");
    }
    printhexchar( numbuf, mspBuf[i] ); 
    EUARTputString(numbuf); EUARTputString(" ");
#endif
    if( mspBuf[i] == 0xFF )
      numOnes ++;
    else
      numZeros++;
  }
  
#if DEBUG_MSP
  EUARTputString( "numzeros: " );
  printhex( numbuf, numZeros );
  EUARTputString(numbuf);
  EUARTputString( " numones: " );
  printhex( numbuf, numOnes );
  EUARTputString( numbuf );
  EUARTputString( "\n" );
#endif
  
  if( numZeros >= numOnes ) {
    EUARTputString("MSP is cleared; flagging normal mode.\n");
    doUpdateBoot = 0; // turn off auto-update boot only if first byte is cleared
  }
  ///////////////////REMOVE FOR PRODUCTION CODE/////////////////////////////////
  ///////////////////REMOVE FOR PRODUCTION CODE/////////////////////////////////
  ///////////////////REMOVE FOR PRODUCTION CODE/////////////////////////////////
  // this line is commented out per bug 11 do NOT uncomment without opening a bug to comment it
  // doUpdateBoot = 0;  // force normal update for DEBUG ONLY!!!!!
  // this line is commented out per bug 11 do NOT uncomment without opening a bug to comment it
  ///////////////////REMOVE FOR PRODUCTION CODE/////////////////////////////////
  ///////////////////REMOVE FOR PRODUCTION CODE/////////////////////////////////
  ///////////////////REMOVE FOR PRODUCTION CODE/////////////////////////////////

/* ksteele 11/14/08: commenting out clearing of MSP to force normal boots; clearing of the MSP
 * should be conducted by usermode applications.  This change addresses a problem where the AC
 * power bounces and the user unknowingly ends up in K2/RFS2.     

  // clear the MSP semaphore page
  for( i = 0; i < MSP_LENGTH; i += (NAND_FLASH_PAGE_MAIN_SIZE * NAND_FLASH_PAGE_PER_BLOCK) ) {
    if( 0 != NAND_Erase((MSP_OFFSET + i) / NAND_FLASH_PAGE_MAIN_SIZE) ) {
      EUARTputString( "MSP block erase failed! FLASH integrity is questionable.\n" );
      EUARTputString( "Attempting to continue boot...\n" );
    }
  }
*/
  
  return doUpdateBoot;
}


///The Main Function
int main()
{
  volatile unsigned int delay;
  char	*pCmdLine;
  char    *pMem;
  int 	i;
  int	bootOption;
  int	fileSize;
  
#ifdef __PLL_CHOICE__
  int mpll[3];
  int MFI,MFN,MFD;
  unsigned long mpll_value,fout;
  unsigned long presc,bclkdiv,ipdiv,fref;
  int fout_trial;
  signed long deviation;
  unsigned long temp;
  int redundant = 0;	//stat data
  char c;
  char  freq = '7';//default setting, mpll 266,fclk 266, bclk 88.	
#endif
  
  unsigned char pageBuffer[512];
  unsigned char oobBuffer[16];
  int doUpdateBoot = 1;
  
  char numBuf[20];
  int stilltouched = 1;

  char c;
  int j, k;
  int foundString;
  int retryCount;
  int match;
  unsigned char replBuf[256];
  static unsigned char v38str[] = "HVRSAAAAAAAAAAAAAAAACAMAAA==";
  static unsigned char v37str[] = "HVRSAAAAAAAAAAAAAAAABwMAAA==";
  int versionCode = 0;
  
  void (*theKernel)(int zero, int arch, u32 params);
  unsigned long exec_at = KERNEL_START_ADDRESS;
  unsigned long parm_at = DRAM_BASE + 0x100;
  // MACH_TYPE_MX21, defined in mach-types.h, is 851...can't figure out how to get a macro to insert into the ASM parens.
  // which is 0x353, but you can't move tha in there in a single shot so it's constructed...
  unsigned long machine_type = 0x353;  // mx21 ads machine type from linux kernel
  
#if 0
  // led testing -- basic sanity testing if everything is broken
  for( j = 0; j < 4; j++ ) {
    for( i = 0; i < 1000000; i++ ) {
      *((unsigned int *)0x1001521C) = 0x0;
    }
    for( i = 0; i < 200000; i++ ) {
      *((unsigned int *)0x1001521C) = 0x00004000;
    }
  }
#endif
  
  init();
  
  //////////// mute speakers so you don't hear any weird noises while the system boots
  disable_speaker_startup();
  
  /////////////////// setup initial port settings!
  // LCD_PWRON   K19  1 for on  KPCOL2
  // CPU_AUD_ENA K18  1 for on  KPCOL4
  _reg_CRM_PCCR1 |= 0x40000000;
  *((unsigned short *)0x10008000) = 0x0000; // KPCR
  *((unsigned short *)0x10008004) = 0xFF00; // KDDR
  *((unsigned short *)0x10008006) = 0xFF00; // KPDR
  *((unsigned short *)0x10008002) |= 0x400; // KPSR

  // force PWM output pin high to get instantaneous LCD response on boot (3.8 hardware, does nothing on 3.7 hardware)
  _reg_GPIO_GIUS(GPIOE) |= 0x20; // set to GPIO function
  _reg_GPIO_GPR(GPIOE)  &= 0xFFFFFFDF; // select primary function
  _reg_GPIO_DDIR(GPIOE) |= 0x20;       // set output
  _reg_GPIO_OCR1(GPIOE) |= 0x0C00; // 11 means data register
  _reg_GPIO_DR(GPIOE) = 0x20;
  
  //Set this for NAND Flash work rightly
  //_reg_CRM_PCDR0=(_reg_CRM_PCDR0&0xffff0fff)|0x00007000;
  // now that core is at 350 MHz, init to 350/11
  _reg_CRM_PCDR0 &= 0xFFFF0FFF;
  _reg_CRM_PCDR0 |= 0x0000C000;
  for(i=0;i<1000;i++);

  /////////////// UART initialization
  MX21_UartSetting();
  MX21_InitInternalUART();


#if DEBUG
  EUARTputString("peep!\n" );
#endif
#if DEBUG_DRIVE  // this code loop debugs drive strength problems on the MX21
  i = 0;
  while(1) {
    i = i % 0x1FF8000;
    if( (i % 1024) == 0)
      EUARTputString("." );
    if( (i % 16384) == 0) {
      printhex(numBuf, i + 0xC0008000);
      EUARTputString(numBuf);
    }
    // swap this value with char, 0xFF combos to debug drive strength issues
    *(unsigned long *)(i + 0xC0008000) = (unsigned long) (0xFFFFFFFF);
    i+=4;
  }
#endif
  
#if 0  // sanity check big vs. little endian configuration if necessary
  long one= 1;
  if( !(*((char *)(&one))) ) {
    EUARTputString("bigendian\n" );
  } else {
    EUARTputString("lilendian\n" );
  }
#endif
  
  //////////////////// scan for bad blocks
  BBscan();
  
#if 0  // this prints the bad block table for debug
  char numBuf[20];
  for( i = 0; i < MAX_BLOCKS; i++ ) {
    EUARTputString("Block 0x");
    printhex(numBuf, i);
    EUARTputString(numBuf);
    EUARTputString(" offset = 0x" );
    printhex(numBuf, BB_TBL_PTR[i]);
    EUARTputString(numBuf);
    EUARTputString("\n");
  }
#endif

  /////////////////// banners
  EUARTputString("\nWelcome to the Chumby Ironforge Linux Loader v1.2 (bunnie@chumby.com)\n");
  EUARTputString("Brought to you by Chumby Industries, Inc.\n\n");
  EUARTputString("        --> Happy hacking! <--\n\n");


  //////////////////////////////
  /////// check versions
  ///////    aggressive retry is coded in because hardware is stabilizing at this time
  ///////    sometimes truncated or corrupted strings make their way back to the MX21
  ///////    exact match is required for the version string to pass
  EUARTputString("Checking CP hardware version information...\n");
  CP_UartSetting();
  CP_InitInternalUART();

  foundString = 0;
  retryCount = 0;
  k = 0;
  do {
    do {
      ///////////// wake up the CP to start the version checking
      CPputString("!!!!!!!!!!!!!!!!!!!");
      c = '\0';
      i = 0;
      while((c != '?') && (i < 20)) {
	CPputString("!");
	c = CPgetData();
	CPputData(c);
	i++; 
      }
      if( c == '?' ) 
	EUARTputString("CP woke up!\n");
      else {
	EUARTputString("CP didn't wake up! Kicking the UART and trying again...\n" );
	CPresetUART();
	while((c != '?') && (i < 20)) {
	  CPputString("!");
	  c = CPgetData();
	  CPputData(c);
	  i++; // put a timeout here
	}
	if( c != '?' ) {
	  EUARTputString("CP doesn't exist or will not wake up, cannot read hardware version...\n");
	}
      }

      ///////////// now issue the command to read the version number
      c = '\0';
      i = 0; // counts only valid characters
      j = 0; // counts everything
      CPputString("!!!!HWVR");
      while( (j < 36) && (c != 0xD) ) {
	c = CPgetData();
	if( c == '?' ) {// ignore synch chars
	  CPputString("!!!!HWVR");
	  j++;
	  continue;
	}
	replBuf[i] = c;
	i++;
	j++;
      }
      i--;  // eat up the 0xd character
      replBuf[i] = '\0';
      EUARTputString("Response from CP: ");
      EUARTputString(replBuf);
      EUARTputString("\n");
      k++;
    } while( (i < 28) && (k < 50) ); // try up to 50 times to get a valid length HWVR

    // version 3.7: HVRSAAAAAAAAAAAAAAAABwMAAA==
    // version 3.8: HVRSAAAAAAAAAAAAAAAACAMAAA==
    versionCode = 0x00;
    match = 1;
    for( i = 0; i < 28; i++ ) {
      if( replBuf[i] != v38str[i] ) // compare against a v38 string
	match = 0;
    }
    if( match == 1 ) {
      versionCode = 0x38;
      foundString = 1;
    } else {
      match = 1;
      for( i = 0; i < 28; i++ ) {
	if( replBuf[i] != v37str[i] ) // compare against a v37 string
	  match = 0;
      }
      if( match == 1 ) {
	versionCode = 0x37;
	foundString = 1;
      }
    }
    retryCount++;
  } while( (foundString == 0) && retryCount < 50 ); // try up to 50 times to get a valid HWVR code

  if( versionCode == 0x38 )
    EUARTputString("Version code for 3.8 found\n");
  else if( versionCode == 0x37 )
    EUARTputString("Version code for 3.7 found\n");
  else {
    EUARTputString("Version code not in match table!!!! Assuming 3.7/0.0 version code, pray and move on...\n");
    versionCode == 0x37;
  }
  EUARTputString("\n");

  
  ///////////////////////////
  //////////  clears the LCD and sets up initial screen images
  setupBuffers();
#if DEBUG
  EUARTputString("setupbuffers\n");
#endif
  blankScreen(versionCode);
#if DEBUG
  EUARTputString("blankscreen\n");
#endif

  touchpanel_init(); // this inits the touch panel 
  
  ///////////////////////
  //////////// check MSP settings for safe mode determination
  doUpdateBoot = checkMSPsetting();
#if DEBUG
  EUARTputString("MSPsetting\n");
#endif

  //////////////////////
  ////////////  do safe mode logic
  if( touchpanel_touched() ) {
    // behavior: if touchpanel is touched and held for longer than TIMEOUT
    // force an update boot. Otherwise, continue with what we were going
    // to do before (be it update or normal boot)
    stilltouched = 1;
    EUARTputString("Touchpanel touched...\n");
    displayTouchMsg();        
    delay = 0;
    while( (delay < TIMEOUT) ) {
      delay++;
      if( !touchpanel_touched() )
	stilltouched = 0;
    }

    tsc2100_clear_touched();
    delay = 0;
    while( delay < 30000 ) {
      delay++;
    }
    
    if( touchpanel_touched() ) { // poll at end of touch interval, just once
      doUpdateBoot = 1;  // force update boot
      EUARTputString("Touchpanel held down for entire period; forcing auto-update mode.\n");
      for( i = 0; i < B5_3_SIZE; i++ ) {
	*((unsigned long *) FRAMEBUF_PTR + i) = *((unsigned long *) B5_3_BACK_PTR + i);
      }
    } else {
      doUpdateBoot = 0;
      for( i = 0; i < B4_1_SIZE; i++ ) {
	*((unsigned long *) FRAMEBUF_PTR + i) = *((unsigned long *) B4_1_BACK_PTR + i);
      }
    }
  } else if( doUpdateBoot ) {
    EUARTputString("Autoupdate preference detected, confirming...\n");
    displayBypassMsg();
    delay = 0;
    while( (delay < TIMEOUT) && !touchpanel_touched() ) 
      delay++;
    
    if( delay < TIMEOUT ) {
      // this means we were touched before the timeout happened
      doUpdateBoot = 0;  // force normal mode
      EUARTputString("Touchpanel touched; forcing normal mode.\n");
      for( i = 0; i < B4_1_SIZE; i++ ) {
	*((unsigned long *) FRAMEBUF_PTR + i) = *((unsigned long *) B4_1_BACK_PTR + i);
      }
    } else {
      for( i = 0; i < B5_3_SIZE; i++ ) {
	*((unsigned long *) FRAMEBUF_PTR + i) = *((unsigned long *) B5_3_BACK_PTR + i);
      }
    }
  } 
#if DEBUG
  EUARTputString("done with touchpanel\n");
#endif
  
#if 0  // this is deprecated in 2.6, as we now use ATAGS to pass the command line!
  EUARTputString("Linux command line: " );
  if( doUpdateBoot )
    EUARTputString((char *)cmdLineK2);
  else
    EUARTputString((char *)cmdLineK1);
  EUARTputString("\n");
#endif

  /////////// inform the user as to which mode we're booting into
  if( doUpdateBoot ) {
    EUARTputString( "\n --> Booting into auto-updater mode... <-- \n\n" );
  } else {
    EUARTputString( "\n --> Booting into normal mode... <-- \n\n" );
  }
  
#if DEBUG_BLOCKKILLER // this is the block killer code, I'm used to generate test cases for bad block tolerance on ROMs
  //Block 0x000000EC is bad.
  //Block 0x000000F0 is bad.
  //Block 0x000000F4 is bad.
  NAND_Read_OnePage(0x1F00, pageBuffer, oobBuffer);
  // offset:	0x3C0000  is page 0x1E00
  // page numbers as address
  //  unsigned int address = 0x1DFF;  // case 1: one bad block before 
  //  unsigned int address = 0x1DFE;  // case 2: two bad blocks before 
  //  unsigned int address = 0x1E00;  // case 3: bad block at start of partition
  //  unsigned int address = 0x1E01;  // case 4: bad block into partition
  //  unsigned int address = 0x1E03;  // case 5: two bad blocks in partition (start and mid)
  
  // previously, 0x1F00, 0x1F20, 0x1F60

  /*
    Things to remember:
    * blocks are marked bad by the first PAGE only! (512 bytes) 
    * blocks are bad in 16-k chunks
    * 0x20 pages per block.
  */

  /* 
     0x1800 -- page number of K2
       -- ......xx.x.Sx..x..xx..
       0x1760
       0x1780
       0x17C0
       0x1800 (start, mark as good)
       0x1820
       0x1880
       0x18E0
       0x1900
     0x6C00 -- page number of K1
       -- ...........S......xx.xx.xx...
       0x6CE0
       0x6D00
       0x6D40
       0x6D60
       0x6DA0
       0x6DC0
   */
  if(memPoker(0x1760)) 
    EUARTputString( "error\n" );
  if(memPoker(0x1780)) 
    EUARTputString( "error\n" );
  if(memPoker(0x17C0)) 
    EUARTputString( "error\n" );
  if(memPoker(0x1820)) 
    EUARTputString( "error\n" );
  if(memPoker(0x1880)) 
    EUARTputString( "error\n" );
  if(memPoker(0x18E0)) 
    EUARTputString( "error\n" );
  if(memPoker(0x1900)) 
    EUARTputString( "error\n" );

  if(memPoker(0x6CE0)) 
    EUARTputString( "error\n" );
  if(memPoker(0x6D00)) 
    EUARTputString( "error\n" );
  if(memPoker(0x6D40)) 
    EUARTputString( "error\n" );
  if(memPoker(0x6D60)) 
    EUARTputString( "error\n" );
  if(memPoker(0x6DA0)) 
    EUARTputString( "error\n" );
  if(memPoker(0x6DC0)) 
    EUARTputString( "error\n" );
  
  
  EUARTputString( "16k erase-blocks were killed.\n" );
  while(1) {
    ; // now wait.
  }
#endif
  
  ///////////////// load up the kernel into RAM
  EUARTputString("Copy Kernel to Memory\n");
  
  CopyKernelToMem(doUpdateBoot);
  
  EUARTputString("\nJump To Kernel\n");
  mx21_module_init();
  
#if 0		
  for( i = 0; i < 0x20000; i+= 0x1000 ) {
    EUARTputString( "(in)sanity: 0xC00..." );
    printhex(numBuf, 0x8000 + i * 4);
    EUARTputString(numBuf);
    EUARTputString(": ");
    printhex(numBuf, *(((unsigned long *) KERNEL_START_ADDRESS) + i));
    EUARTputString(numBuf);
    EUARTputString(" ");
    printhex(numBuf, *(((unsigned long *) KERNEL_START_ADDRESS) + i + 1));
    EUARTputString(numBuf);
    EUARTputString("\n");
  }
#endif		

  /////////// before we jump, leave an image around for the kernel to initialize its framebuffer
  if( doUpdateBoot ) {
    for( i = 0; i < B5_3_SIZE * 4; i++ ) {
      *((unsigned char *) (KERNEL_LOGO_PTR + i)) = *(unsigned char *)(B5_3_BACK_PTR + i);  // wants to be 5_4 but no space for the buffer in flash
    }
  } else {
    for( i = 0; i < B4_2_SIZE * 4; i++ ) {
      *((unsigned char *) (KERNEL_LOGO_PTR + i)) = *(unsigned char *)(B4_2_BACK_PTR + i); 
    }
  }

  ///////////// make sure the clock settings are right before going into the kernel...
#if USE12M_XTAL
  _reg_CRM_SPCTL0 =  0x80003000; // 288 MHz from a 12 M xtal
  _reg_CRM_SPCTL1 |= 0x00000040; // select proper filter style
  _reg_CRM_CSCR  |= 0x00020000; // select the HS xtal
  _reg_CRM_CSCR  |= 0x00400000; // restart                                                                                   
  while( _reg_CRM_CSCR & 0x00400000 )
    ;
#endif
#if USE16M_XTAL
  _reg_CRM_SPCTL0 =  0x80002400; // 288 MHz from a 12 M xtal
  _reg_CRM_SPCTL1 |= 0x00000040; // select proper filter style
  _reg_CRM_CSCR  |= 0x00020000; // select the HS xtal
  _reg_CRM_CSCR  |= 0x00400000; // restart                                                                                   
  while( _reg_CRM_CSCR & 0x00400000 )
    ;
#endif

  ///////////////////// setup kernel command line arguments through ATAGS
  // command line is set in the following setup_tags call
  // (and possibly overridden by the auto-bunzipper if you're using zImage)
  setup_tags((void *)parm_at, doUpdateBoot, versionCode);                    /* sets up parameters */
  theKernel = (void (*)(int, int, u32))exec_at; /* set the kernel address */

  ////////// do the jump to kernel!
  theKernel(0, machine_type, parm_at);    /* jump to kernel with registers set per C calling convention */

  return 0;  // this never actually gets executed
}
//>>>>>Body
