/*************************************************************************
  bl.c

  Copyright (c) Chumby Industries 2007                bunnie@chumby.com
  Based on code by Freescale Suzhou Design Center
  Dual MPL/GPL2 license

  "The contents of this file are subject to the Mozilla Public License
  Version 1.1 (the "License"); you may not use this file except in
  compliance with the License. You may obtain a copy of the License at
  http://www.mozilla.org/MPL/

  Software distributed under the License is distributed on an "AS IS"
  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
  License for the specific language governing rights and limitations
  under the License.

  The Original Code is from the Freescale MX21 ADS SDK.

  The Initial Developer of the Original Code is the Freescale Suzhou 
  Design Center. Portions created by Chumby Industries are Copyright (C)
  2007 Chumby Industries. All Rights Reserved.

  Contributor(s): bunnie@chumby.com

  Alternatively, the contents of this file may be used under the terms
  of the BSD license (the  "[BSD] License"), in which case the
  provisions of [BSD] License are applicable instead of those
  above.  If you wish to allow use of your version of this file only
  under the terms of the [BSD] License and not to allow others to use
  your version of this file under the MPL, indicate your decision by
  deleting  the provisions above and replace  them with the notice and
  other provisions required by the [BSD] License.  If you do not delete
  the provisions above, a recipient may use your version of this file
  under either the MPL or the [BSD] License."

  "Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
      * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
      * Neither the name of the Chumby Industries nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.
  
  THIS SOFTWARE IS PROVIDED BY Chumby Industries ``AS IS'' AND ANY
  EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL Chumby Industries BE LIABLE FOR ANY
  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."

*************************************************************************/

// Note that the compiled size of this code must be 1,536 bytes or less.
// This code must share a 2048-byte buffer in the MX21 with the boot.S 
// initialization page.

// the order of execution is:
// boot.S -> bl.c -> 2bl.c -> linux kernel

//<<<<<Include
#define IO_ADDRESS 
#include "mx2.h"
#include "mx2_nand.h"
//>>>>>Include

//<<<<<< Private Macro
///2BL Run SDRAM address, bl will copy 2bl.bin to this address 
#define START_SDRAM_ADDRESS  0xc0001000

///NAND Flash Start Page of 2BL
#define START_NAND_ADDRESS   (0x00000800/NAND_FLASH_PAGE_MAIN_SIZE)

///NAND Flash End Page of 2BL
#define START_END_ADDRESS    (0x00040000/NAND_FLASH_PAGE_MAIN_SIZE)
//>>>>>> Private Macro

///copy 2bl to SDRAM and jump to 2bl.

#define USE26M_XTAL 0
#define USE12M_XTAL 0
#define USE16M_XTAL 1
int main()
{
    unsigned char* buff = (char*) START_SDRAM_ADDRESS;
      
    int i, j;

#if 0    
    // set LED to on; assume: PCCR0 has GPIO clocks turned on in boot.S
    _reg_GPIO_DDIR(GPIOC) = 0x00004000;
    _reg_GPIO_GIUS(GPIOC) = 0x00004000;
    _reg_GPIO_DR(GPIOC) = 0x00004000;
    _reg_GPIO_OCR1(GPIOC) = 0x30000000;
#endif
    _reg_GPIO_DR(GPIOC) = 0x00004000; // turn on LED

#if 0
    // led testing
    for( j = 0; j < 4; j++ ) {
      for( i = 0; i < 1000000; i++ ) {
	*((unsigned int *)0x1001521C) = 0x0;
      }
      for( i = 0; i < 1000000; i++ ) {
	*((unsigned int *)0x1001521C) = 0x00004000;
      }
    }
#endif

#if USE26M_XTAL
    _reg_CRM_MPCTL0 = 0x809b1872;  // 350 MHz from a 26 M xtal
    
    _reg_CRM_CSCR = 0x17190A07;
    _reg_CRM_CSCR = 0x17390A07;

    while (_reg_CRM_CSCR & 0x00200000)
      ; // wait for stabilization

    _reg_CRM_SPCTL0 = 0x80811446;
    _reg_CRM_CSCR  |= 0x00020000; // select the HS xtal
    _reg_CRM_CSCR  |= 0x00400000; // restart                                                                                   
    while( _reg_CRM_CSCR & 0x00400000 )
      ;

#elif USE12M_XTAL

    _reg_CRM_MPCTL0 = 0x800b3807;  // 350 MHz from a 12 M xtal
    
    _reg_CRM_CSCR = 0x17190A07;
    _reg_CRM_CSCR = 0x17390A07;

    while (_reg_CRM_CSCR & 0x00200000)
      ; // wait for stabilization

    _reg_CRM_SPCTL0 =  0x80003000; // 288 MHz from a 12 M xtal
    _reg_CRM_SPCTL1 |= 0x00000040; // select proper filter style
    _reg_CRM_CSCR  |= 0x00020000; // select the HS xtal
    _reg_CRM_CSCR  |= 0x00400000; // restart                                                                                   
    while( _reg_CRM_CSCR & 0x00400000 )
      ;

#elif USE16M_XTAL

    _reg_CRM_MPCTL0 =  0x803f283c;  // 350 MHz from a 16 M xtal, 10x 60/63
    _reg_CRM_MPCTL1 |= 0x00000040; // select proper filter style
    
    _reg_CRM_CSCR = 0x17190A07;
    _reg_CRM_CSCR = 0x17390A07;

    while (_reg_CRM_CSCR & 0x00200000)
      ; // wait for stabilization

    _reg_CRM_SPCTL0 =  0x80002400; // 288 MHz from a 12 M xtal
    _reg_CRM_SPCTL1 |= 0x00000040; // select proper filter style
    _reg_CRM_CSCR  |= 0x00020000; // select the HS xtal
    _reg_CRM_CSCR  |= 0x00400000; // restart                                                                                   
    while( _reg_CRM_CSCR & 0x00400000 )
      ;

#else
    //MFI	MFN	MFD	PD	UPCTL0/MPCTL0	actual   	actual
    //   	   	   	  	             	tolerance	frequency
    //===	===	===	==	=============	=========	=========
    //10	282	413	0	0x019d291a	0.000070%	350.000244
    //_reg_CRM_MPCTL0 = 0x819d291a;  // 350 MHz
    _reg_CRM_MPCTL0 = 0x8044282f;  // 350 MHz, 47/68 MFN/MFD, PFD enabled
    // _reg_CRM_MPCTL1 |= 0x00000040;  // new code

    _reg_SYS_GPCR = 0;

    // presc = 000 (div by 1)
    // presc = 011 (div by 4)
    // usb_div = 101 (default)
    // sd_cnt = 11 (default)
    // spll_restart = 0
    // mpll_restart = 1
    // ssi2_sel = 1
    // ssi1_sel = 1
    // fir_sel = 0
    // sp_sel = 0
    // mcu_sel = 0
    // bclkdiv = 0010  (div by 3, sdram freq = 116 MHz)
    // ipdiv = 1
    // osc26mdiv1p5 = 0
    // osc26m_dis = 1
    // fpm_en = 1
    // spen = 1
    // mpen = 1
    //             v
    // 0001 0111 00X1 1000 0000 1010 0000 1111
    // 0001 0111 00X1 1000 0011 1110 0000 1111  // slow bclk
    _reg_CRM_CSCR = 0x17180A0F; // dont restart pll yet, divider at /4
    //_reg_CRM_CSCR = 0x77183E0F; // dont restart pll yet, divider at /4, slow bclk
    
    _reg_CRM_CSCR = 0x17380A0F; // restart it now
    //_reg_CRM_CSCR = 0x77383E0F; // restart it now // slow bclk, extclk
    while (_reg_CRM_CSCR & 0x00200000)
      ; // wait for stabilization
#endif

    _reg_CRM_CSCR &= 0x1FFFFFFF; // set divider to /1

    // initialize the NANDFC clcok as 266/8
    //_reg_CRM_PCDR0 |= 0x7000;
    // now that core is at 350 MHz, init to 350/11
    //    _reg_CRM_PCDR0 &= 0xFFFF0FFF;
    _reg_CRM_PCDR0 |= 0x0000F000;
    //    for(i=0;i<10000;i++);  // wait a moment.

#if 0
    // led testing
    for( j = 0; j < 4; j++ ) {
      for( i = 0; i < 1000000; i++ ) {
	*((unsigned int *)0x1001521C) = 0x0;
      }
      for( i = 0; i < 1000000; i++ ) {
	*((unsigned int *)0x1001521C) = 0x00004000;
      }
    }
#endif

    // As it's in the first block of NAND flash, no bbc & ecc needed. 
    for(i = START_NAND_ADDRESS; i < START_END_ADDRESS; i++) {
#if 0
      for( j = 0; j < 50000; j++ ) {
	*((unsigned int *)0x1001521C) = 0x0;
      }
      for( j = 0; j < 50000; j++ ) {
	*((unsigned int *)0x1001521C) = 0x00004000;
      }
#endif
      NAND_Read_OnePage( i, buff, (char*)0);
      buff += NAND_FLASH_PAGE_MAIN_SIZE;
    }

#if 0
    // led testing
    for( j = 0; j < 4; j++ ) {
      for( i = 0; i < 500000; i++ ) {
	*((unsigned int *)0x1001521C) = 0x0;
      }
      for( i = 0; i < 500000; i++ ) {
	*((unsigned int *)0x1001521C) = 0x00004000;
      }
    }
#endif

    return START_SDRAM_ADDRESS;
    // return value is jump target in r0 (value in r0 by compiler convention)
}


//>>>>>Body

