/*
       boot.S
       February 2007 -- port to Ironforge -- bunnie@chumby.com

       This file is part of the chumby bootloader.
       Copyright (c) Chumby Industries, 2007
       All Rights Reserved.

       Redistribution and use in source and binary forms, with or without
       modification, are permitted provided that the following conditions are met:
           * Redistributions of source code must retain the above copyright
             notice, this list of conditions and the following disclaimer.
           * Redistributions in binary form must reproduce the above copyright
             notice, this list of conditions and the following disclaimer in the
             documentation and/or other materials provided with the distribution.
           * Neither the name of the Chumby Industries nor the
             names of its contributors may be used to endorse or promote products
             derived from this software without specific prior written permission.
       
       THIS SOFTWARE IS PROVIDED BY Chumby Industries ``AS IS'' AND ANY
       EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
       WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
       DISCLAIMED. IN NO EVENT SHALL Chumby Industries BE LIABLE FOR ANY
       DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
       (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
       LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
       ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
       (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
       SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

// note that the assembled size of this code must be 512 bytes or less
// it must share a 2048-byte buffer with 1536 bytes from bl.c
// the order of execution is:
// boot.S -> bl.c -> 2bl.c -> linux kernel

	// size is stored in location 0x0C0000FC
	.equ SOURCE, 0xDF003200
	.equ TARGET, 0xC0000000
 	
	.global	_start
_start:

	.word   0xE59FF000
	.word   0
	.word	( _start2 - 4 )
	.word	0
	.word	( _start2 - 4 )
_loop1:	b	_loop1
	nop
	nop
	nop
	nop

_start2:
         ldr r1,=0x10000000  // initialize the AIPI registers (some voodoo here)
         ldr r3,=0x00040304
         str r3,[r1]
         
         ldr r1,=0x10020000  // initialize the AIPI registers
         ldr r3,=0x00000000
         str r3,[r1]

         ldr r1,=0x10000004  // initialize the AIPI registers
         ldr r3,=0xFFFBFCFB
         str r3,[r1]

         ldr r1,=0x10020004  // initialize the AIPI registers
         ldr r3,=0xFFFFFFFF
         str r3,[r1]        

        //# Explicitly set MPLL 266MHz
	// for low spurs shoot for the largest possible divider?
	//Clock Source Control Register CSCR 0x10027000
	//MPLL Control Register 0 MPCTL0 0x10027004
	//MPLL Control Register 1 MPCTL1 0x10027008
	//SPLL Control Register 0 SPCTL0 0x1002700C
	//SPLL Control Register 1 SPCTL1 0x10027010
	//Oscillator 26M Register OSC26MCTL 0x10027014
	//Peripheral Clock Divider Register 0 PCDR0 0x10027018
	//Peripheral Clock Divider Register 1 PCDR1 0x1002701C
	//Peripheral Clock Control Register 0 PCCR0 0x10027020
	//Peripheral Clock Control Register 1 PCCR1 0x10027024

        ldr r3,=0x007b1C73 // for 266 MHz

        ldr r1,=0x10027004 // mpll register
        str r3,[r1]       
	  
        ldr r1,=0x10027000  // cscr register

        ldr r3,=0x77000207  // 0111 0111 0000 0000 0000 0010 0000 0111
	// Prescaler divide by 4  <-- need to change that
	// SPLL divide by 4 for USB clock
	// no shutdown, don't restart plls, clock sources from internal mpy
	// system bus clock is BLCK divide by 1
	// perclk is HCLK / 2
	// enable both PLLs and frequency premult
	// OSC26M is enabled
	// NEW
	// presc = 000 (div by 1)
	// usb_div = 101 (default)
	// sd_cnt = 11 (default)
	// spll_restart = 0
	// mpll_restart = 1
	// ssi2_sel = 1
	// ssi1_sel = 1
	// fir_sel = 0
	// sp_sel = 0
	// mcu_sel = 0
	// bclkdiv = 0010  (div by 3, sdram freq = 116 MHz)
	// ipdiv = 1
	// osc26mdiv1p5 = 0
	// osc26m_dis = 1
	// fpm_en = 1
	// spen = 1
	// mpen = 1
	//             v
	// 0001 0111 00X1 1000 0000 1010 0000 1111
        str r3,[r1]
	
        // bits 31-8 should be 1 -- 7-0 should be: 1100 0001
	// selects 512b nand flash
	// selects 8 bit nand flash
	// selects CSD0
	// selects CS3
        ldr r1,=0x10027814
        ldr r3,=0xFFFFFFC1
        str r3,[r1]          

preloop:	
	ldr   r0,=0x000000
	// delay 200 us for micron DRAM
	add   r3, r1, r0
	add   r0, r0, #4
	teq   r0, #0x100000
	beq   preloop

//comment Set Precharge Command
         ldr r1,=0xDF000000
	 ldr r3,=0x9219C268
         str r3,[r1]    
         
//comment Issue Precharge all Command
        LDR  r3, =0xC0200000 
        LDR  r2, [r3] 

//comment Set AutoRefresh Command
	LDR r3, =0xA219C268
        STR  r3, [r1] 

       //  Issue AutoRefresh Command 
        LDR  r3, =0xC0000000 
        LDR  r2, [r3] 
        LDR  r2, [r3] 
        LDR  r2, [r3] 
        LDR  r2, [r3] 
        LDR  r2, [r3] 
        LDR  r2, [r3] 
        LDR  r2, [r3] 
        LDR  r2, [r3]
	nop
	nop
	nop
	nop

//comment Set Mode Register
        LDR  r3, =0xB219C268 //=0xB2120300 
        STR  r3, [r1]
	
	// values for x32, 16MBx2 RAM, CL=2
	// IAM  = 1, 24:11 is the mode command
	// with {A11:0,BA1:0} as the format
	// 31	27   23   19   15   11   7    3
	// 0000 0000 0100 0100 0110 0000 0000 0000
	// 0000 0000 0100 0110 0110 0000 0000 0000
	// C0446000
	// C0466000
	
        LDR  r3, =0xC0466000 // CL3
        LDR  r2, [r3] 
	nop
	nop
	nop
	nop
	
	// derived for Hynix HY57V561620LTP devices
	// 13 row address lines, 9 column address lines
	// SDE = 1 SMODE = 000 SP = 0 row = 10 col = 01 IAM = 1 DSIZ = 10
	// SREFR = 11 PWDT = 10 CI = 00 SCL = 11 SRP = 0 SRCD = 11
	// SRC = 1001
	// BA = 2, RA = 12, CA = 9, DQ =4, IAM = 1
	// 1000 0010 0001 1010 1110 0011 0011 1001
	// ----> power down enabled! <-----
	// 0x821AE339
	// 1000 0010 0001 1010 1100 0011 0011 1001
	// ----> power down disabled! <-----
	// required due to hardware bug in LM3671/4 regulator circuits
	// they don't deal nicely with power-on transients from the SDRAM
	// 0x821AC339
	LDR  r3, = 0x821AC339
        STR  r3, [r1]
	// issue the command	
        LDR  r3, = 0xC0000000
        LDR  r2, [r3] 

	
// adjust driving strength of I/Os to reduce EMI
		ldr	r3,=0x10027824
		mov r0,#0x00000000   // set it to 3.5 mA (smallest setting) to reduce EMI
		ldr r2,=0x9

_DrivingLoop:
		str	r0,[r3]
		add	r3,r3,#4
		sub	r2,r2,#1
		teq	r2,#0
		beq	_EndDriving
		b	_DrivingLoop
		
_EndDriving:				

	// bl.c counts on this clock being turned on, don't delete this line
	// turn on GPIO and DDIR
	ldr r1,=0x10027020 //pccr0 clock control
	ldr r3,=0x31084803 //defaults plus gpio
	str r3,[r1]

	// the following instructions moved to bl.c
        //PC14
	ldr r1,=0x10015204 //ptcocr1
	ldr r3,=0x30000000
	str r3,[r1]
	ldr r1,=0x10015200 //ptcddir
	ldr r3,=0x00004000
	str r3,[r1]
	ldr r1,=0x10015220 //ptcgius
	str r3,[r1]
//	ldr r1,=0x1001521C //ptcdr --sets the value
//	str r3,[r1]

/*
ledloop:	
//	// turn on LED to see if we are alive, used for early boot debugging
	ldr r1,=0x1001521C //ptcdr --sets the value
	// 0010 0000 0000 0000
	ldr r3,=0x00004000
	str r3,[r1]
	ldr r3,=0x00000000
	str r3,[r1]
	b ledloop
*/
			 
//;*************************************** 
//;*  End of SDRAM and SyncFlash Init    * 
//;*************************************** 

// copy next stage boot code from FLASH to SRAM
copyCodes:
        ldr     r0,=SOURCE
        ldr     r1,=TARGET
        sub     r3,r0,#4
        ldr     r2,[r3]
	ldr     r2,=0x600

_CopyLoop:
        ldr     r3,[r0]
        str     r3,[r1]
        add     r0,r0,#4
        add     r1,r1,#4
        sub     r2,r2,#4
        teq     r2,#0
        beq     _EndCopy
        b       _CopyLoop

_EndCopy:
	
        ldr     r0,=TARGET
        mov     pc,r0

