#!/usr/bin/perl

#       buildGoldLoader.pl
#
#       This file is part of the chumby bootloader builder script.
#       Copyright (c) Chumby Industries, 2007
#
#       The bootloader builder is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       The bootloader builder is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with the Chumby; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

##### buildGoldLoader
#####
##### bunnie - 5/16/2006 - v1.0
##### this is a script that assembles all the binary portions together for
##### everything in a ROM image, PSP and below, inclusive.
##### flash map sizes and lengths are hard-coded in the code below
##### 
##### bunnie - 1/3/2007 - v2.0
##### blew away the test image, MSP, and other items to accommodate a full-sized note image
##### the test image is now using a different method--runs a script from RFS2 on a dongle, 
##### not out of hard code memory, so it's okay to get rid of it.
#####
##### bunnie - 3/21/2007 - v2.1
##### added the "bypass" note segment
##### bunnie - 6/8/2007 - v2.2
##### added secondary boot screen segment and reworked naming to reflect new spec for art assets

use strict;
use warnings;

my $numArgs = $#ARGV + 1;
if($numArgs < 9) {
    print "Usage: \n";
    print "buildGoldLoader <1st page> <bl> <2bl> <4_1> <5_1> <5_2> <4_2> <5_3> <output>";
    exit 0;
}

open OFILE, ">$ARGV[8]"
    or die "\nCan't open $ARGV[8] for writing: $!\n";
binmode OFILE;

# constants below are from FLASH map
my $firstPgEnd = 0x200;
my $blEnd = 0x800;
my $twoBlEnd =   0x10000;
my $logoEnd =    0x35800;    # this is 4_1.bin
my $noteEnd =    0x5B000;    # this is 5_1.bin
my $bypassEnd =  0x80800;    # this is 5_2.bin
my $bootEnd =    0xA6000;    # this is 4_2.bin
my $specialEnd = 0xCB800;    # this is 5_3.bin
#my $mspEnd = 0x44000;
#my $tstEnd = 0x5A800;
#my $tstImgEnd = 0x80000;
#my $tstImgEnd = 0x5AA00;
#my $pspEnd = 0xC0000;

# constants below are size limits based on the lesser of FLASH or SDRAM restrictions
# hence we specify the size constraint tests differently than the
# FLASH rom region boundaries above
my $firstpagesize = 0x200;
my $blsize = 0x600;
# size constraint is lesser of SDRAM footprint and FLASH footprint
# in this case, it is SDRAM that constrains us (for twoblsize)
my $twoblsize = 0xF000;
my $logosize =    0x25800;
my $notesize =    0x25800;   # A800 (was)
my $bypasssize =  0x25800; 
my $bootsize =    0x25800;
my $specialsize = 0x25800;

#my $mspsize = 0x4000;
#my $tstsize = 0x16800; # 92k to fit my test code!
#my $tstimgsize = 0x25800;
#my $tstimgsize = 0x100; # for now let's leave this out to accomodate space for bad blocks
#my $pspsize = 0x40000;

fillFile($ARGV[0], $firstPgEnd, $firstpagesize);
fillFile($ARGV[1], $blEnd - $firstPgEnd, $blsize);
fillFile($ARGV[2], $twoBlEnd - $blEnd, $twoblsize);
fillFile($ARGV[3], $logoEnd - $twoBlEnd, $logosize);
fillFile($ARGV[4], $noteEnd - $logoEnd, $notesize);
fillFile($ARGV[5], $bypassEnd - $noteEnd, $bypasssize);
fillFile($ARGV[6], $bootEnd - $bypassEnd, $bootsize);
fillFile($ARGV[7], $specialEnd - $bootEnd, $specialsize);
#fillFile($ARGV[5], $mspEnd - $noteEnd, $mspsize);
#fillFile($ARGV[6], $tstEnd - $mspEnd, $tstsize);
#fillFile($ARGV[7], $tstImgEnd - $tstEnd, $tstimgsize);
#fillFile($ARGV[8], $pspEnd - $tstImgEnd, $pspsize);

close OFILE
    or die "Can't close $ARGV[8]: $!\n";

exit 0;

######################################################
# fillFile subroutine
######################################################

sub fillFile {
  #args: file name, the FLASH rom size for the region, and the expected file size
  #expeted file size may be less than FLASH rom size due to SDRAM limitations
  #e.g., the SDRAM region is smaller than the FLASH rom image area
  my ($srcfile, $srcFlashSize, $srcRamSize) = @_;
  my $buffer;
  
  open INFILE, $srcfile
      or die "\nCan't open $srcfile for reading: $!\n";
  binmode INFILE;
  my $fsize = -s $srcfile;
  print "$srcfile size: $fsize\n";
  if( $fsize > $srcRamSize ) {
      print "$srcfile code size must be less than $srcRamSize bytes, aborting.\n";
      close OFILE
	  or die "Can't close $ARGV[8]: $!\n";
      exit 1;
  }
  
  my $readLength = 0;
  $readLength = read (INFILE, $buffer, 262144) # read in (up to) 256k chunks, write
      and print OFILE $buffer;	# exit if read or write fails
  die "Problem copying: $!\n" if $!;

  my $fillChar = "\xff";
  while($readLength < $srcFlashSize) {
      print OFILE $fillChar;
      die "Problem filling: $!\n" if $!;
      $readLength++;
  } 
}


#random notes

#foreach $argnum (0 .. $#ARGV) {
#   print "$ARGV[$argnum]\n";
#}

