#include "common.h"
#include "tht_memory_map_defines.h"

# define PCDR1_PERDIV3_POS       16
# define PCDR1_PERDIV3_MASK      (0x3f << PCDR1_PERDIV3_POS)
# define PCDR1_PERDIV3(x)        (((x) << PCDR1_PERDIV3_POS) & \
                                  PCDR1_PERDIV3_MASK)

/******************************************************************************
 Other function definitions
******************************************************************************/

void EnableICache (void)
{

  asm( "mrc p15,0,r0,c1,c0,0" );
  asm( "mov r2, #0x00001000" );
  asm( "orr r0,r2,r0" );
  asm( "mcr p15,0,r0,c1,c0,0" );

} /* EnableICache */

void DisableICache (void)
{

  asm( "mrc p15,0,r0,c1,c0,0" );
  asm( "mov r2, #0xFFFFEFFF" );
  asm( "and r0,r2,r0" );
  asm( "mcr p15,0,r0,c1,c0,0" );


} /* DisableICache */

void SysInit(void)
{
	/* Initialize PLL and clocks here - setup the MPLL for 266.0000537MHz
	   (PD = 0; MFI = 7; MFN = 115; MFD = 123) */
  //	*(p_uint32_t)CRM_MPCTL0 = 0x007B1C73;
  //	*(p_uint32_t)CRM_MPCTL1 = 0x00000040;
	
	/* Set up the SPLL for 287.9999978Mhz operation */
  //	*(p_uint32_t)CRM_SPCTL0 = 0x03B02227;
	
	/* Set USBDIV=5; SD_CNT=3; BCLKDIV=2; IPDIV=1; HCLK will be 133MHz */
  //	*(p_uint32_t)CRM_CSCR = 0x17000607;		

	/* Enable in PCCR0: HCLK_DMA, LCDC, LCDC_PIXCLK, DMA_EN, GPIO_EN */
  

	*(p_uint32_t)CRM_PCCR0 |= 0x44042800;

	
	/* Peripheral Clock Divider Register 1 (PCDR1)
	   PERDIV3 (PCDR1[21:16]) = 7 :Sets PERCLK3 to SYSCLK/8 (33.25MHz) */
	//	*(p_uint32_t) (CRM_PCDR1) |= 0x03070F0F;
	
} /* SysInit */

void LCDC_Disable(void)
{


	/* Clear PCCR0[18] and PCCR0[26] to disable HCLK and PERCLK3 to LCDC */

	*(p_uint32_t)(CRM_PCCR0) &= ~0x04040000;

} /* LCDC_Disable */

void LCDC_Enable(void)
{

	/* Set PCCR0[18] and PCCR0[26] to enable HCLK and PERCLK3 to LCDC */
	*(p_uint32_t)(CRM_PCCR0) |= 0x04040000;
	

} /* LCDC_Enable */	

/******************************************************************************
Data Image 3.5" FG030530DNSWBGT1 - MX21 ADS Initialization for MX21 TO2.1
******************************************************************************/
void LCDC_Init(int versionCode, unsigned long *lcdBufLoc)
{
	// Clear PtA GIUS and GPR to use LCD signals
	*(p_uint32_t)(GPIOA_GIUS) = 0x00000000;
	*(p_uint32_t)(GPIOA_GPR) = 0x00000000;
	
	*(p_uint32_t)(GPIOF_GIUS) = 0x00000000;
	*(p_uint32_t)(GPIOF_GPR) = 0x00000000;

	// Set LCDC SSAR to location of frame buffer in SDRAM
	*(p_uint32_t)(LCDC_LSSAR) = (unsigned long) lcdBufLoc; 	

	// Landscape QVGA: 320 x 240
	
	// Set size of display to 320 x 240
	//   XMAX (LSR[25:20]) = 320/16 = 20
	//   YMAX (LSR[9:0])   = 240
	*(p_uint32_t)(LCDC_LSR) = 0x028001E0;

	// Set virtual page width to width of display
	//   VPW (LVPWR[9:0]) = 320/2 = 160
	*(p_uint32_t)(LCDC_LVPWR) = 0x00000140;
		
	// Cursor is not used in this application - clear registers to be safe
	*(p_uint32_t)(LCDC_LPOR) = 0x00000000;
	*(p_uint32_t)(LCDC_LCPR) = 0x00000000;
	*(p_uint32_t)(LCDC_LCWHBR) = 0x00000000;
	*(p_uint32_t)(LCDC_LCCMR) = 0x00000000;
	
	// disable graphics window (we're only using the main screen)
	*(p_uint32_t)(LCDC_LGWCR) = 0x00000000; 

	// contrast control
	// 0000 0000 1010 1001 0000 0011 1100 0000
	*(p_uint32_t)(LCDC_LPCCR) = 0x00A903FF;

	// Disable self-refresh by clearing bit LRMCR[9]
	*(p_uint32_t)(LCDC_LRMCR) = 0x00000000;
	
	// Setup LCDC panel configuration
	//   TFT      (LPCR[31])    = 1  		:TFT display
	//   COLOR    (LPCR[30])    = 1  		:Color display
	//   PBSIZ    (LPCR[29:28]) = 00 		:Active matrix fixed at 16bpp
	
	//   BPIX     (LPCR[27:25]) = 101		:16pp in memory
	//   PIXPOL   (LPCR[24])    = 0  		:Pixels are active low
	
	//   FLMPOL   (LPCR[23])    = 0  		:VSYNC is active high
	//   LPPOL    (LPCR[22])    = 0  		:HSYNC is active high
	//   CLKPOL   (LPCR[21])    = 0  		:LSCLK is active on positive edge
	//   OEPOL    (LPCR[20])    = 0  		:OE is active high
	
	//   SCLKIDLE (LPCR[19])    = 1  		:Enable LSCLK when VSYNC is idle
	//   END_SEL  (LPCR[18])    = 0  		:Image in memory is in little endian format
	//   SWAP_SEL (LPCR[17])    = 0  		:Ignored for big endian format
	//   REV_VS   (LPCR[16])    = 0  		:Vertical scan in normal direction
	
	//   ACDSEL   (LPCR[15])    = 0  		:Ignored for TFT display
	//   ACD      (LPCR[14:8])  = 0000000 	:Ignored for TFT display

	//   SCLKSEL  (LPCR[7])     = 1  		:Enable OE and LSCLK when no data output
	//   SHARP    (LPCR[6])     = 0  		:Disable Sharp signals
	//   PCD      (LPCR[5:0])   = 000100  	:Pixel Clock = LSCLK = 33.25MHz/5 = 6.65MHz (DI demo board is @ 5.26MHz)
	//                          = 000111  	:V and H timing calculated with Pixel Clock = LSCLK = 33.25MHz/8 = 4.15MHz, but less noise at higher LSCLK
	EUARTputString( "setting up VGA timings\n" );
	*(p_uint32_t)(CRM_PCDR1) &= ~PCDR1_PERDIV3_MASK;  // setup PCDR1 to / 7 so we can have a faster clock...
	*(p_uint32_t)(CRM_PCDR1) |= PCDR1_PERDIV3(6);
	  
	// vga timings
	*(p_uint32_t)(LCDC_LPCR) = 0xCAC80082;	
	
	// If we're not using a Sharp panel, clear this register to be safe
	*(p_uint32_t)(LCDC_LSCR) = 0x00000000;
	
	// Configure horizontal LCD timing
	*(p_uint32_t)(LCDC_LHCR) = 0x78008272;
	  
	// Configure vertical LCD timing
	*(p_uint32_t)(LCDC_LVCR) = 0x0C000A20;
		
	// Setup DMA for main frame buffer
	//   BURST (LDCR[31])    = 0 :Dynamic burst length
	//   HM    (LDCR[20:16]) = 4 :DMA high mark is set to 4
	//   TM    (LDCR[4:0])   = 15 :Set DMA trigger mark to 15
	*(p_uint32_t)(LCDC_LDCR) = 0x0004000F;
} // LCDC_init
