#undef CONFIG_ARCH_IMX
#define CONFIG_ARCH_IMX21

#include "hardware26.h"
#include "tsc2100.h"

typedef unsigned long u32;
typedef unsigned short u16;

#define SPIDEV              2

#define TSC2100_ADC_DEFAULT (TSC2100_ADC_RES(TSC2100_ADC_RES_12BITP)   | \
                             TSC2100_ADC_AVG(TSC2100_ADC_4AVG)         | \
                             TSC2100_ADC_CL(TSC2100_ADC_CL_1MHZ_12BIT) | \
                             TSC2100_ADC_PV(TSC2100_ADC_PV_500us)      | \
                             TSC2100_ADC_AVGFILT_MEAN) 

#define CLK32 32768

void ndelay( unsigned int delay ) {
  int i;
  for( i = 0; i < delay; i++ ) {
    ;
  }
}

void disable_speaker_startup(void) {
  imx_gpio_mode( GPIO_PORTKP | (4 + 8) | GPIO_OUT );
  imx_gpio_write( GPIO_PORTKP | (4 + 8), 1 );
} 

void imx_gpio_mode(int gpio_mode)
{
	unsigned int pin = gpio_mode & GPIO_PIN_MASK;
	unsigned int port = (gpio_mode & GPIO_PORT_MASK) >> GPIO_PORT_POS;
	unsigned int ocr = (gpio_mode & GPIO_OCR_MASK) >> GPIO_OCR_POS;
	unsigned int tmp;

        if (port != (GPIO_PORTKP >> GPIO_PORT_POS)) {

            /* Pullup enable */
            if(gpio_mode & GPIO_PUEN)
		PUEN(port) |= (1<<pin);
            else
		PUEN(port) &= ~(1<<pin);
            
            /* Data direction */
            if(gpio_mode & GPIO_OUT)
		DDIR(port) |= 1<<pin;
            else
		DDIR(port) &= ~(1<<pin);
            
            /* Primary / alternate function */
            if(gpio_mode & GPIO_AF)
		GPR(port) |= (1<<pin);
            else
		GPR(port) &= ~(1<<pin);
            
            /* use as gpio? */
            if( ocr == 3 )
		GIUS(port) |= (1<<pin);
            else
		GIUS(port) &= ~(1<<pin);
            
            /* Output / input configuration */
            /* FIXME: I'm not very sure about OCR and ICONF, someone
             * should have a look over it
             */
            if(pin<16) {
		tmp = OCR1(port);
		tmp &= ~( 3<<(pin*2));
		tmp |= (ocr << (pin*2));
		OCR1(port) = tmp;
                
		if( gpio_mode &	GPIO_AOUT )
                    ICONFA1(port) &= ~( 3<<(pin*2));
		if( gpio_mode &	GPIO_BOUT )
                    ICONFB1(port) &= ~( 3<<(pin*2));
            } else {
		tmp = OCR2(port);
		tmp &= ~( 3<<((pin-16)*2));
		tmp |= (ocr << ((pin-16)*2));
		OCR2(port) = tmp;
                
		if( gpio_mode &	GPIO_AOUT )
                    ICONFA2(port) &= ~( 3<<((pin-16)*2));
		if( gpio_mode &	GPIO_BOUT )
                    ICONFB2(port) &= ~( 3<<((pin-16)*2));
            }

            if (gpio_mode & GPIO_IRQ_MASK) {
                int cfg;
                cfg = (gpio_mode & GPIO_IRQ_CFG_MASK) >> GPIO_IRQ_CFG_POS;

                /* Mask the interrupt */
                IMR(port) &= ~(1 << pin);

                if (pin < 16) {
                    tmp = ICR1(port);
                    tmp &= ~(3 << (pin * 2));
                    tmp |= cfg << (pin * 2);
                    ICR1(port) = tmp;
                } else {
                    tmp = ICR2(port);
                    tmp &= ~(3 << ((pin - 16) * 2));
                    tmp |= cfg << ((pin - 16) * 2);
                    ICR2(port) = tmp;
                }
            }

        } else { /* PORT KP */

            if (gpio_mode & GPIO_OCEN) {
                if (pin < 8) {
		  // don't do anything, we're not in the kernel
                } else {
                    KPP_KPCR |= (1 << pin);
                }
            }
            if (gpio_mode & GPIO_OUT) {
                KPP_KDDR |= (1 << pin);
            } else {
                KPP_KDDR &= ~(1 << pin);
            }
        }
}

void imx_gpio_write(int mode,  int val)
{
    int pin  = (mode & GPIO_PIN_MASK);
    int port = (mode & GPIO_PORT_MASK) >> GPIO_PORT_POS;

    if (port != (GPIO_PORTKP >> GPIO_PORT_POS)) {
        if (val) {
            DR(port) |= (1 << pin);
        } else {
            DR(port) &= ~(1 << pin);
        }
    } else {
        if (val) {
            KPP_KPDR |= (1 << pin);
        } else {
            KPP_KPDR &= ~(1 << pin);
        }
    }
}

static inline void spi_wait_for_idle( void )
{
    while (SSP_CTRL_REG(SPIDEV) & SSP_XCH)
        ndelay(50);
}


static inline void spi_flush_fifo( void )
{
    volatile int junk;
    while (SSP_INT_REG(SPIDEV) & SSP_INT_RR)
        junk = SSP_RX_REG(SPIDEV);
}


static inline void spi_read_fifo(int* rxdata, int rxlen)
{
    while (rxlen > 0)
        while (SSP_INT_REG(SPIDEV) & SSP_INT_RR) {
            *rxdata++ = SSP_RX_REG(SPIDEV) & 0xffff;
            if (--rxlen <= 0)
                break;
        }
}


static inline void spi_write_zeros(int zeros)
{
    while (zeros-- > 0)
        SSP_TX_REG(SPIDEV) = 0;
}


void spi_init( void )
{
    unsigned int rate;
    unsigned int rate_div;
    unsigned int i;

    SSP_RESET_REG(SPIDEV) = 1;
    for( i = 0; i < 70000; i ++ ) {
      ; // do nothing
    }

    PCDR1_PERDIV2(34);

    PCCR1 |= PCCR1_CSPI3_EN; // enable PERCLK to CSPI3
    SSP_CTRL_REG(SPIDEV) = (
        SSP_RATE_DIV4   |
        SSP_MODE_MASTER |
        SSP_SS_POL_LOW  |
        SSP_ENABLE      |
        SSP_PHA1        |
        SSP_POL0        |
        SSP_WS(16));

    spi_flush_fifo();
}


static void spi_exit( void )
{
    SSP_CTRL_REG(SPIDEV) = SSP_DISABLE;
    PCCR1 &= ~PCCR1_CSPI3_EN; // disable PERCLK to CSPI3
}


static inline int spi_write( int regaddr, int regdata )
{
    unsigned long flags;

    //    spin_lock_irqsave(&tsc2100_devdata->lock, flags);
    spi_wait_for_idle();
    spi_flush_fifo();
    SSP_TX_REG(SPIDEV) = regaddr | TSC2100_WRITE;
    SSP_TX_REG(SPIDEV) = regdata;
    SSP_CTRL_REG(SPIDEV) |= SSP_XCH;
    //    spin_unlock_irqrestore(&tsc2100_devdata->lock, flags);
    return 0;
}


static inline int spi_read( int regaddr, int* rxdata, int rxlen )
{
    int junk;
    unsigned long flags;

    //    spin_lock_irqsave(&tsc2100_devdata->lock, flags);
    if ( rxlen > 7 )  
        rxlen = 7;
    spi_wait_for_idle();
    spi_flush_fifo();
    SSP_TX_REG(SPIDEV) = regaddr | TSC2100_READ;
    spi_write_zeros(rxlen);
    SSP_CTRL_REG(SPIDEV) |= SSP_XCH;
    spi_read_fifo(&junk, 1);
    spi_read_fifo(rxdata, rxlen);
    //    spin_unlock_irqrestore(&tsc2100_devdata->lock, flags);
    return rxdata[0];
}


static inline int tsc2100_regread(int regnum)
{
    int reg;
    return spi_read(regnum, &reg, 1);
}


static inline void tsc2100_regwrite(int regnum, int value)
{
    spi_write( regnum, value );
}

static inline void disable_speaker( void )
{
    imx_gpio_write( GPIO_PORTKP | (4 + 8), 1 );
}
void gpio_init( void )
{
    imx_gpio_mode( 18 | GPIO_PORTE | GPIO_AF | GPIO_IN );
    imx_gpio_mode( 21 | GPIO_PORTE | GPIO_AF | GPIO_OUT );
    imx_gpio_mode( 22 | GPIO_PORTE | GPIO_AF | GPIO_OUT );
    imx_gpio_mode( 23 | GPIO_PORTE | GPIO_AF | GPIO_OUT );
    imx_gpio_mode( 19 | GPIO_PORTE | GPIO_PF | GPIO_IN | GPIO_PUEN | GPIO_GPIO );

#if 0
    imx_gpio_mode( PC23_PF_SSI1_CLK );
    imx_gpio_mode( PC22_PF_SSI1_TX );
    imx_gpio_mode( PC21_PF_SSI1_RX );
    imx_gpio_mode( PC20_PF_SSI1_FS );
    imx_gpio_mode( PC27_PF_SSI2_CLK );
    imx_gpio_mode( PC26_PF_SSI2_TX );
    imx_gpio_mode( PC25_PF_SSI2_RX );
    imx_gpio_mode( PC24_PF_SSI2_FS );
#endif

    imx_gpio_mode( GPIO_PORTKP | (4 + 8) | GPIO_OUT );
    disable_speaker();
}


/***********************************************************************
 * TouchScreen Driver Routines
 ***********************************************************************/
#if 0
static void tsc2100_touchscreen_report(struct tsc2100_data *tsc2100_ts, 
                                       int x, int y, int p, int pendown)
{
	input_report_abs(tsc2100_ts->inputdevice, ABS_X, x);
	input_report_abs(tsc2100_ts->inputdevice, ABS_Y, y);
	input_report_abs(tsc2100_ts->inputdevice, ABS_PRESSURE, p);
	input_report_key(tsc2100_ts->inputdevice, BTN_TOUCH, pendown);
	input_sync(tsc2100_ts->inputdevice);
}
#endif

#if 0
static void tsc2100_touchscreen_setup(struct device *dev)
{
	struct tsc2100_data *tsc2100_ts = dev_get_drvdata(dev);

	tsc2100_ts->inputdevice->name = "tsc2100_ts";
	tsc2100_ts->inputdevice->evbit[0] = BIT(EV_KEY) | BIT(EV_ABS);
	tsc2100_ts->inputdevice->keybit[LONG(BTN_TOUCH)] |= BIT(BTN_TOUCH);
	input_set_abs_params(tsc2100_ts->inputdevice, 
                         ABS_X, X_AXIS_MIN, X_AXIS_MAX, 0, 0);
	input_set_abs_params(tsc2100_ts->inputdevice, 
                         ABS_Y, Y_AXIS_MIN, Y_AXIS_MAX, 0, 0);
	input_set_abs_params(tsc2100_ts->inputdevice, 
                         ABS_PRESSURE, PRESSURE_MIN, PRESSURE_MAX, 0, 0);
	input_register_device(tsc2100_ts->inputdevice);
	printk("Chumby TI-TSC2100 TouchScreen Driver initialized\n");
}
#endif

int tsc2100_pendown( void )
{
    return tsc2100_regread(TSC2100_REG_STATUS) & TSC2100_STATUS_DAVAIL;
}

void tsc2100_clear_touched() {
    u32 values[4];
    
    // must read all registers before pendown is cleared
    spi_read(TSC2100_REG_X, &values[0], 4);
}


static void tsc2100_readdata(struct tsc2100_data *devdata, 
                             struct tsc2100_ts_event *ts_data)
{
    int z1,z2;
    u32 values[4];

    if (!tsc2100_pendown())
        return;

    /* Read X, Y, Z1 and Z2 
     */
    spi_read(TSC2100_REG_X, &values[0], 4);
    ts_data->x=values[0];
    ts_data->y=values[1];
    z1=values[2];
    z2=values[3];

#if 0
    /* Calculate Pressure 
     */
    if ((z1 != 0) && (ts_data->x!=0) && (ts_data->y!=0)) 
      ts_data->p = ((ts_data->x * (z2 -z1) / z1));  // no division in the bootloader
    else
        ts_data->p=0;
#endif

    #ifdef TSC2100_DEBUG
    printk( "%s/%s(): X=%d, Y=%d, Z1=%d, Z2=%d, p=%d\n",
            __FILE__, __FUNCTION__, values[0], values[1], values[2], values[3], 
            ts_data->p );
    #endif
}


void tsc2100_touchscreen_enable()
{
    tsc2100_regwrite(TSC2100_REG_RESETCTL, 0xbb00);
    
    /* PINTDAV is data available only 
     */
    tsc2100_regwrite(TSC2100_REG_STATUS, 0x4000);

    /* TSC2100-controlled conversions, 12-bit samples, continuous X,Y,Z1,Z2 
     * scan mode, average (mean) 4 samples per coordinate, 1 MHz internal 
     * conversion clock, 500 usec panel voltage stabilization delay
     */
    tsc2100_regwrite(TSC2100_REG_ADC, 
              TSC2100_ADC_DEFAULT | TSC2100_ADC_PSM | TSC2100_ADC_ADMODE(0x2));
}


static void tsc2100_touchscreen_disable(struct tsc2100_data *devdata)
{
    /* stop conversions and power down 
     */
    tsc2100_regwrite(TSC2100_REG_ADC, 0x4000);
}

#if 0
static void touchscreen_interrupt_main(struct tsc2100_data *devdata, 
                                       int isTimer, struct pt_regs *regs)
{
    unsigned long flags;
    struct tsc2100_ts_event ts_data;

    spin_lock_irqsave(&devdata->lock, flags);

    if (tsc2100_pendown()) {
        /* PEN is down, this is either the first PEN down (!isTimer), or
         * the PEN still down 10ms later (isTimer) -- in either case, report
         * the PEN down event
         */
        devdata->pendown = 1;
        tsc2100_readdata(devdata, &ts_data);
        tsc2100_touchscreen_report(devdata, ts_data.x, ts_data.y, ts_data.p, 1);
        mod_timer(&(devdata->ts_timer), jiffies + HZ / 100);
    } 
    else if (devdata->pendown > 0 && devdata->pendown < 3) {
        /* time the 10ms timer events, after 3, report PEN up event
        */
        mod_timer(&(devdata->ts_timer), jiffies + HZ / 100);
        devdata->pendown++;
    } else {
        if (devdata->pendown) 
        {
            devdata->pendown = 0;
            tsc2100_touchscreen_report(devdata, 0, 0, 0, 0);
        }

        /* okay, PEN up event has been reported, now change back to falling
         * edge to report new PEN down events -- 
         * make sure no data is missed after set_irq_type()
         */
        set_irq_type(PENIRQ, IRQT_FALLING);
        if (tsc2100_pendown()) {
            tsc2100_readdata(devdata, &ts_data);
            mod_timer(&(devdata->ts_timer), jiffies + HZ / 100);
        } 
    }
    spin_unlock_irqrestore(&devdata->lock, flags);
}
#endif
