///@ingroup    NAND_BOOTLOADER
///@file uart.c
///@brief Uart operator function
///
///@author 
///@version $Version$

#ifndef _STANDALONE_
#define _STANDALONE_
#endif 

#define IO_ADDRESS 
#include "mx2.h"

#define TXFE_MASK   0x4000  // Tx buffer empty
#define RDR_MASK    0x0001  // receive data ready
//#define BIR_115200	1151
#define BIR_115200	837


typedef unsigned short u16;
/* put wemi init code here */

///Config Mx21 Uart1 Clock
void CP_UartSetting(void)
{

 	int temp;

    	_reg_CRM_PCCR0 |= 0x00000004; // enable my clock
 	
	// software reset
	_reg_UART_UCR2(UART_3) = 0x4026;
	
	//write 1 to RXDMUXSEL = 1
	_reg_UART_UCR3(UART_3) |= 0x4;

	// M17 - UART3_RXD (input)  PE9
	// L19 - UART3_TXD (output) PE8
	//=================================================================
	// Set up GPIO/IOMUX for UART_3  
	_reg_GPIO_GIUS(GPIOE) &= 0xFFFFFCFF;	// clear bits 8/9 of GIUS_E
	_reg_GPIO_GPR(GPIOE) &= 0xFFFFFCFF;	// clear bits 8/9 of GPR_E
	_reg_GPIO_DDIR(GPIOE) |= 0x100; // bit 8 is 1 for output
	_reg_GPIO_DDIR(GPIOE) &= 0xFFFFFDFF;
	//=================================================================
	
}

void CPresetUART(void) {
  int i;

  _reg_UART_UCR2(UART_3) &= 0xFFFE;  // trigger the reset bit
  for( i = 0; i < 100000; i++ ) {
    // do nothing
  }
  _reg_UART_UCR2(UART_3) |= 0x1;

  CP_UartSetting();  // now reset all the registers
  CP_InitInternalUART();

}


///Config MX21 Uart1 As 115200-B-N-1
void CP_InitInternalUART(void)
{
#if 0
	//software reset
        // 0101 0000 0010 0110
	_reg_UART_UCR2(UART_3) = 0x5026;
	// 0000 0000 0000 0001
	_reg_UART_UCR1 (UART_3)= 0x1;
	// 0000 0000 0000 0100
	_reg_UART_UCR3(UART_3) = 0x4;
	_reg_UART_UCR4 (UART_3)= 0x8000;
#endif
	_reg_UART_UCR2(UART_3) = 0x4027;
	_reg_UART_UCR1 (UART_3)= 0x221;
	_reg_UART_UCR3(UART_3) = 0x4;
	_reg_UART_UCR4 (UART_3)= 0x8000;
	
	//=================================================================
	// Set up reference freq divide for UART module 
	// MX2 only support 16MHz output
	// PerCLK1(31.99998691MHz)/UFCR[RFDIV] (2)= 15.99999673MHz
	// ** 101 is div by 2, so 22.15 MHz @ 266 MHz
	// ** so this is 21.875 MHz @ 350 MHz ( 350 MHz / 8 / 2 )
	_reg_UART_UFCR(UART_3) = 0x0A01; 	// CKIH = PerCLK1 set to div by 1 (UART 1) // was a01

	//=================================================================

	//clear loopback bit
	_reg_UART_UTS(UART_3) = 0x0060; // was 0

	//reference frequency is 16MHz
	//BIR_115200 = 1151
	//NUM  = 1151+1		DENOM = 9999+1
	//thus NUM/DENOM = 0.1152 --> gives baud rate: 115200		
	// ** (838/10000 * 22.15 MHz) / 16 = 116010
	// (38400 * 16) / 21.875 MHz  = num/denom = 0.028086, num = 10, denom = 356
	// so UBIR = 9, UBMR = 355
	//	 _reg_UART_UBIR(UART_3) = 837;
	_reg_UART_UBIR(UART_3) = 9; // was 9

	// set for 38400 comms
	_reg_UART_UBMR(UART_3) = 355; // was 355

}

///Check if rx FIFO Ready
///@return	@li 0:		not ready
///		@li not 0:	Ready 
char CPdataReady()
{
    return _reg_UART_USR2(UART_3) & RDR_MASK;    // check RDR bit
}

///Send Data 
///@param data data will be sent
void CPputData(char data)
{
    while (!(_reg_UART_USR2(UART_3) & TXFE_MASK));    // wait until TXFE bit set
    _reg_UART_UTXD(UART_3) = (u16)data;
}

///Get Data
///@return data recieved by UART
char CPgetData()
{
  unsigned long int i = 0;
  
  // hard code a timeout, even if it returns incorrect data, because you really
  // want the bootloader to not get stuck in a wait loop
  while ((!CPdataReady()) && (i < 100000)) 
    i++;   // wait until RDR bit set
  return (char)_reg_UART_URXD(UART_3);
}

///Put data as Hex format
///@param 	data 	data will be sent 
void CPputHex(unsigned char data)
{
    unsigned char  d;

    // print first digit
    d = data >> 4;
    if (d > 9)
        d += 55;
    else
        d += '0';
    CPputData(d);

    // print second digit
    d = data & 0xF;
    if (d > 9)
        d += 55;
    else
        d += '0';
    CPputData(d);
}

///put string to uart
///@param 	line		string will be sent by Uart
void CPputString(char *line)
{
    while (*line)
    {
        CPputData(*(line++));
    }
}
