/********** Macro definition for SW level**********************************/
//#define PRINT
//#define Tahiti
//#define FPGA
//#define NOLM

//#define UartTest	//Uart test only (5 polling times)
//#define UsbTest	//USB test (100000 polling times + I2C Address + SCL clock change same as host model)
//#define Production	//Production (100000 polling times) with I2C setting same as reality

#define TIMEOUT_VALUE 100000
#ifdef UartTest
	#define Tahiti
	#define TIMEOUT_VALUE 5	
#endif

#ifdef UsbTest
	#define Tahiti
	#define TIMEOUT_VALUE 100000
#endif

#ifdef Production
	#define Tahiti
	#define TIMEOUT_VALUE 100000
#endif

#ifdef FPGA
#define UARTonly
#define NewSW
#define TIMEOUT_VALUE 100000
#endif


//#define PACKET_SIZE 32

#define BIR_115200	1151
#define BIR_57600	575


#define ErrorFree 	0x0

// Command Packet Format: Header(2)+Address(4)+Format(1)+ByteCount(4)+Data(4) 
#define	ReadHeader	0x0101
#define WriteHeader 0x0202
#define PCHeader 	0x0303
#define WriteFile 	0x0404
#define ErrorHeader	0x0505
#define CSFHeader	0x0606
#define HWConfigHeader	0x0707
#define ExecuteHeader	0x0808

#define ProductionPart 	0x12343412
#define DevelopmentPart 0x56787856



#ifdef Tahiti

#define NFCstart	0xDF003000
#define NFCend		0xDF003E1C
#define WEIMstart	0xDF001000
#define WEIMend		0xDF001030
#define MemoryStart 0xC0000000
#define MemoryEnd	0xD3FFFFFF
#define PLLstart	0x10027000
#define PLLend		0x10027020
#define SDRAMCstart	0xDF000000
#define SDRAMCend	0xDF001018
#define SysCstart	0x10027804
#define SysCend		0x10027850
#endif

#ifdef FPGA
#define NFCstart	0x80000000
#define NFCend		0x81000000
#define WEIMstart	0x81000000
#define WEIMend		0x82000000
#define MemoryStart 0x82000000
#define MemoryEnd	0x88000000
#define PLLstart	0x83000000
#define PLLend		0x84000000
#define SDRAMCstart	0x83100000
#define SDRAMCend	0x83101018
#define SysCstart	0x83027804
#define SysCend		0x83027850
#endif



/*=====OTG=====*/
#define BIT0 	0x00000001
#define BIT1 	0x00000002
#define BIT2 	0x00000004
#define BIT3 	0x00000008
#define BIT4	0x00000010
#define BIT5 	0x00000020
#define BIT6	0x00000040
#define BIT7	0x00000080
#define BIT8	0x00000100
#define BIT9 	0x00000200
#define BIT10 	0x00000400
#define BIT11 	0x00000800
#define BIT12 	0x00001000
#define BIT13 	0x00002000
#define BIT14 	0x00004000
#define BIT15 	0x00008000
#define BIT16 	0x00010000
#define BIT17 	0x00020000
#define BIT18 	0x00040000
#define BIT19 	0x00080000
#define BIT20 	0x00100000
#define BIT21 	0x00200000
#define BIT22 	0x00400000
#define BIT23 	0x00800000
#define BIT24 	0x01000000
#define BIT25 	0x02000000
#define BIT26 	0x04000000
#define BIT27 	0x08000000
#define BIT28 	0x10000000
#define BIT29 	0x20000000
#define BIT30 	0x40000000
#define BIT31 	0x80000000

#define ENABLE  0x1
#define DISABLE 0x0

#define DELAY_VALUE 0x10000


#define EP_OUT_DIR       0
#define EP_IN_DIR        1
#define EP_NO_STALL      0
#define EP_STALL         1
#define EP_NO_SETUP      0
#define EP_SETUP         1

#define EP0OUT	BIT0
#define EP0IN	BIT1
#define EP1OUT	BIT2
#define EP1IN	BIT3
#define EP2OUT	BIT4
#define EP2IN	BIT5
#define EP3OUT	BIT6
#define EP3IN	BIT7
#define EP4OUT	BIT8
#define EP4IN	BIT9
#define EP5OUT	BIT10
#define EP5IN	BIT11
#define EP6OUT	BIT12
#define EP6IN	BIT13
#define EP7OUT	BIT14
#define EP7IN	BIT15
#define EP8OUT	BIT16
#define EP8IN	BIT17
#define EP9OUT	BIT18
#define EP9IN	BIT19
#define EP10OUT	BIT20
#define EP10IN	BIT21
#define EP11OUT	BIT22
#define EP11IN	BIT23
#define EP12OUT	BIT24
#define EP12IN	BIT25
#define EP13OUT	BIT26
#define EP13IN	BIT27
#define EP14OUT	BIT28
#define EP14IN	BIT29
#define EP15OUT	BIT30
#define EP15IN	BIT31


#define EP0 0x0
#define EP1 0x1
#define EP2 0x2

/*====i2c defines==== */
#ifdef UsbTest
#define I2C_DEV_ADDR	0x2E  
#else
#define I2C_DEV_ADDR	0x2D
#endif

#define I2C_XCVR_LOWADR         0x02 

/*#define I2C_TEST_SCLK_SCL             0x000c0000 --- as defined by TDI*/
#ifdef UsbTest
#define I2C_TEST_SCLK_SCL 	      0x0C	//for simuation 2000Kbps
#else	
#define I2C_TEST_SCLK_SCL 	     	  0xF0	//configure as 100kHz   /* defined for own purposes, due to byte access */
#endif

#define I2C_TEST_SINGLE_WRITE         0x0
#define I2C_TEST_DOUBLE_WRITE         0x1 
#define I2C_TEST_BLOCK_WRITE          0x2 
#define I2C_TEST_RANDOM_WRITE         0x3 

/********** End of Macro definition for SW level**********************************/



/***************** Type declaration **************************/

/* Typedefs for integer types */
typedef unsigned char   U8;     /* unsigned 8 bit data  */
typedef unsigned short  U16;    /* unsigned 16 bit data */
typedef unsigned int    U32;    /* unsigned 32 bit data */
typedef char        S8;         /* signed 8 bit data  */
typedef short       S16;        /* signed 16 bit data */
typedef int         S32;        /* signed 32 bit data */

typedef U8 *    P_U8;           /* unsigned 8 bit data  */
typedef volatile U8 * VP_U8;
typedef U16 *   P_U16;          /* unsigned 16 bit data */
typedef volatile U16 * VP_U16;
typedef U32 *   P_U32;          /* unsigned 32 bit data */
typedef volatile U32 * VP_U32;	/* volatile unsigned 32 bit data */
typedef S8 *    P_S8;           /* signed 8 bit data  */
typedef S16 *   P_S16;          /* signed 16 bit data */
typedef S32 *   P_S32;          /* signed 32 bit data */

typedef U16     TEXT;           /* 16-bit text data */
typedef P_U16   P_TEXT;         /* 16-bit text data */

typedef U8  BOOL;               /* Boolean, TRUE/FALSE */

typedef void    VOID;           /* void */
typedef void *  P_VOID;         /* pointer to void */

typedef unsigned char	uint8_t;	/* unsigned 8 bit data	*/
typedef unsigned short	uint16_t;	/* unsigned 16 bit data	*/
typedef unsigned int	uint32_t;	/* unsigned 32 bit data	*/
typedef signed char		int8_t;		/* signed 8 bit data	*/
typedef signed short	int16_t;	/* signed 16 bit data	*/
typedef signed int		int32_t;	/* signed 32 bit data	*/

typedef uint8_t *		p_uint8_t;	/* unsigned 8 bit data	*/
typedef uint16_t *		p_uint16_t;	/* unsigned 16 bit data	*/
typedef uint32_t *		p_uint32_t;	/* unsigned 32 bit data	*/
typedef int8_t *		p_int8_t;	/* signed 8 bit data	*/
typedef int16_t *		p_int16_t;	/* signed 16 bit data	*/
typedef int32_t *		p_int32_t;	/* signed 32 bit data	*/

typedef uint16_t		text_t;		/* 16-bit text data 	*/
typedef p_uint16_t		p_text_t;	/* 16-bit text data 	*/

typedef uint8_t			bool_t;		/* Boolean, TRUE/FALSE	*/
/***************** End of Type declaration **************************/


/***************** Tahiti definition **************************/
//;#########################################			
//;# AIPI1                                 #			
//;# $1000_0000 to $1000_0FFF              #			
//;#########################################			
#define AIPI1_BASE_ADDR	      	0x10000000	
#define AIPI1_PSR0	      	(AIPI1_BASE_ADDR+0x00)	//  32bit Peripheral Size Reg 0
#define AIPI1_PSR1	     	(AIPI1_BASE_ADDR+0x04)	//  32bit Peripheral Size Reg 1
#define AIPI1_PAR	      	(AIPI1_BASE_ADDR+0x08)	//  32bit Peripheral Access Reg


     
//
// ;---------------------------------------;
// ; UART1                                 ;
// ; $1000A000 to $1000_A0FF             ;
// ;---------------------------------------;
#define UART1_BASE_ADDR         0x1000A000                
#define UART1_RXDATA            UART1_BASE_ADDR                
#define UART1_TXDATA            (UART1_BASE_ADDR+0x40)                
#define UART1_CR1               (UART1_BASE_ADDR+0x80)                
#define UART1_CR2               (UART1_BASE_ADDR+0x84)                
#define UART1_CR3               (UART1_BASE_ADDR+0x88)                
#define UART1_CR4               (UART1_BASE_ADDR+0x8C)                
#define UART1_FCR               (UART1_BASE_ADDR+0x90)                
#define UART1_SR1               (UART1_BASE_ADDR+0x94)                
#define UART1_SR2               (UART1_BASE_ADDR+0x98)                
#define UART1_ESC               (UART1_BASE_ADDR+0x9C)                
#define UART1_TIM               (UART1_BASE_ADDR+0xA0)                
#define UART1_BIR               (UART1_BASE_ADDR+0xA4)                
#define UART1_BMR               (UART1_BASE_ADDR+0xA8)                
#define UART1_BRC               (UART1_BASE_ADDR+0xAC)                
#define UART1_ONEMS             (UART1_BASE_ADDR+0xB0)                
#define UART1_TS	            (UART1_BASE_ADDR+0xB4) 
//------------------------------------------
//System Control
//------------------------------------------

#define SYS_BASE_ADDR	      	0x10027800	
#define SYS_SIDR	      	(SYS_BASE_ADDR+0x10)	//;  128bit Silicon ID Reg
#define SYS_FMCR	      	(SYS_BASE_ADDR+0x14)	//;  Functional Muxing Control Reg
#define SYS_GPCR	      	(SYS_BASE_ADDR+0x18)	//;  Global Peripheral Control Reg
#define SYS_WBCR	      	(SYS_BASE_ADDR+0x1C)	//;  Well Bias Control Reg
#define SYS_DSCR1	      	(SYS_BASE_ADDR+0x20)	//;  Drive Strength Crtl Reg 1
#define SYS_DSCR2	      	(SYS_BASE_ADDR+0x24)	//;  Drive Strength Crtl Reg 2
#define SYS_DSCR3	      	(SYS_BASE_ADDR+0x28)	//;  Drive Strength Crtl Reg 3
#define SYS_DSCR4	      	(SYS_BASE_ADDR+0x2C)	//;  Drive Strength Crtl Reg 4
#define SYS_DSCR5	      	(SYS_BASE_ADDR+0x30)	//;  Drive Strength Crtl Reg 5
#define SYS_DSCR6	      	(SYS_BASE_ADDR+0x34)	//;  Drive Strength Crtl Reg 6
#define SYS_DSCR7	      	(SYS_BASE_ADDR+0x38)	//;  Drive Strength Crtl Reg 7
#define SYS_DSCR8	      	(SYS_BASE_ADDR+0x3C)	//;  Drive Strength Crtl Reg 8
#define SYS_DSCR9	      	(SYS_BASE_ADDR+0x40)	//;  Drive Strength Crtl Reg 9
#define SYS_DSCR10	      	(SYS_BASE_ADDR+0x44)	//;  Drive Strength Crtl Reg 10
#define SYS_DSCR11	      	(SYS_BASE_ADDR+0x48)	//;  Drive Strength Crtl Reg 11
#define SYS_DSCR12	      	(SYS_BASE_ADDR+0x4C)	//;  Drive Strength Crtl Reg 12
#define SYS_PSCR	      	(SYS_BASE_ADDR+0x50)	//;  Priority Control/select Reg



//;#########################################			
//;# #define GPIO                          #			
//;# $1001_5000 to $1001_5FFF              #			
//;#########################################			
#define GPIOA_BASE_ADDR	       	0x10015000	
#define GPIOA_DDIR	      (GPIOA_BASE_ADDR+0x00)   //  32bit gpio pta data direction reg
#define GPIOA_OCR1	      (GPIOA_BASE_ADDR+0x04)   //  32bit gpio pta output config 1 reg
#define GPIOA_OCR2	      (GPIOA_BASE_ADDR+0x08)   //  32bit gpio pta output config 2 reg
#define GPIOA_ICONFA1	      (GPIOA_BASE_ADDR+0x0C)   //  32bit gpio pta input config A1 reg
#define GPIOA_ICONFA2	      (GPIOA_BASE_ADDR+0x10)   //  32bit gpio pta input config A2 reg
#define GPIOA_ICONFB1	      (GPIOA_BASE_ADDR+0x14)   //  32bit gpio pta input config B1 reg
#define GPIOA_ICONFB2	      (GPIOA_BASE_ADDR+0x18)   //  32bit gpio pta input config B2 reg
#define GPIOA_DR	      (GPIOA_BASE_ADDR+0x1C)   //  32bit gpio pta data reg
#define GPIOA_GIUS	      (GPIOA_BASE_ADDR+0x20)   //  32bit gpio pta in use reg
#define GPIOA_SSR	      (GPIOA_BASE_ADDR+0x24)   //  32bit gpio pta sample status reg
#define GPIOA_ICR1	      (GPIOA_BASE_ADDR+0x28)   //  32bit gpio pta interrupt ctrl 1 reg
#define GPIOA_ICR2	      (GPIOA_BASE_ADDR+0x2C)   //  32bit gpio pta interrupt ctrl 2 reg
#define GPIOA_IMR	      (GPIOA_BASE_ADDR+0x30)   //  32bit gpio pta interrupt mask reg
#define GPIOA_ISR	      (GPIOA_BASE_ADDR+0x34)   //  32bit gpio pta interrupt status reg
#define GPIOA_GPR	      (GPIOA_BASE_ADDR+0x38)   //  32bit gpio pta general purpose reg
#define GPIOA_SWR	      (GPIOA_BASE_ADDR+0x3C)   //  32bit gpio pta software reset reg
#define GPIOA_PUEN	      (GPIOA_BASE_ADDR+0x40)   //  32bit gpio pta pull up enable reg
			
#define GPIOB_BASE_ADDR	       	0x10015100	
#define GPIOB_DDIR	      (GPIOB_BASE_ADDR+0x00)   //  32bit gpio ptb data direction reg
#define GPIOB_OCR1	      (GPIOB_BASE_ADDR+0x04)   //  32bit gpio ptb output config 1 reg
#define GPIOB_OCR2	      (GPIOB_BASE_ADDR+0x08)   //  32bit gpio ptb output config 2 reg
#define GPIOB_ICONFA1	      (GPIOB_BASE_ADDR+0x0C)   //  32bit gpio ptb input config A1 reg
#define GPIOB_ICONFA2	      (GPIOB_BASE_ADDR+0x10)   //  32bit gpio ptb input config A2 reg
#define GPIOB_ICONFB1	      (GPIOB_BASE_ADDR+0x14)   //  32bit gpio ptb input config B1 reg
#define GPIOB_ICONFB2	      (GPIOB_BASE_ADDR+0x18)   //  32bit gpio ptb input config B2 reg
#define GPIOB_DR	      (GPIOB_BASE_ADDR+0x1C)   //  32bit gpio ptb data reg
#define GPIOB_GIUS	      (GPIOB_BASE_ADDR+0x20)   //  32bit gpio ptb in use reg
#define GPIOB_SSR	      (GPIOB_BASE_ADDR+0x24)   //  32bit gpio ptb sample status reg
#define GPIOB_ICR1	      (GPIOB_BASE_ADDR+0x28)   //  32bit gpio ptb interrupt ctrl 1 reg
#define GPIOB_ICR2	      (GPIOB_BASE_ADDR+0x2C)   //  32bit gpio ptb interrupt ctrl 2 reg
#define GPIOB_IMR	      (GPIOB_BASE_ADDR+0x30)   //  32bit gpio ptb interrupt mask reg
#define GPIOB_ISR	      (GPIOB_BASE_ADDR+0x34)   //  32bit gpio ptb interrupt status reg
#define GPIOB_GPR	      (GPIOB_BASE_ADDR+0x38)   //  32bit gpio ptb general purpose reg
#define GPIOB_SWR	      (GPIOB_BASE_ADDR+0x3C)   //  32bit gpio ptb software reset reg 
#define GPIOB_PUEN	      (GPIOB_BASE_ADDR+0x40)   //  32bit gpio ptb pull up enable reg
			
#define GPIOC_BASE_ADDR	       	0x10015200	
#define GPIOC_DDIR	      (GPIOC_BASE_ADDR+0x00)   //  32bit gpio ptc data direction reg
#define GPIOC_OCR1	      (GPIOC_BASE_ADDR+0x04)   //  32bit gpio ptc output config 1 reg
#define GPIOC_OCR2	      (GPIOC_BASE_ADDR+0x08)   //  32bit gpio ptc output config 2 reg
#define GPIOC_ICONFA1	      (GPIOC_BASE_ADDR+0x0C)   //  32bit gpio ptc input config A1 reg
#define GPIOC_ICONFA2	      (GPIOC_BASE_ADDR+0x10)   //  32bit gpio ptc input config A2 reg
#define GPIOC_ICONFB1	      (GPIOC_BASE_ADDR+0x14)   //  32bit gpio ptc input config B1 reg
#define GPIOC_ICONFB2	      (GPIOC_BASE_ADDR+0x18)   //  32bit gpio ptc input config B2 reg
#define GPIOC_DR	      (GPIOC_BASE_ADDR+0x1C)   //  32bit gpio ptc data reg
#define GPIOC_GIUS	      (GPIOC_BASE_ADDR+0x20)   //  32bit gpio ptc in use reg
#define GPIOC_SSR	      (GPIOC_BASE_ADDR+0x24)   //  32bit gpio ptc sample status reg
#define GPIOC_ICR1	      (GPIOC_BASE_ADDR+0x28)   //  32bit gpio ptc interrupt ctrl 1 reg
#define GPIOC_ICR2	      (GPIOC_BASE_ADDR+0x2C)   //  32bit gpio ptc interrupt ctrl 2 reg
#define GPIOC_IMR	      (GPIOC_BASE_ADDR+0x30)   //  32bit gpio ptc interrupt mask reg
#define GPIOC_ISR	      (GPIOC_BASE_ADDR+0x34)   //  32bit gpio ptc interrupt status reg
#define GPIOC_GPR	      (GPIOC_BASE_ADDR+0x38)   //  32bit gpio ptc general purpose reg
#define GPIOC_SWR	      (GPIOC_BASE_ADDR+0x3C)   //  32bit gpio ptc software reset reg 
#define GPIOC_PUEN	      (GPIOC_BASE_ADDR+0x40)   //  32bit gpio ptc pull up enable reg
			
#define GPIOD_BASE_ADDR	       	0x10015300	
#define GPIOD_DDIR	      (GPIOD_BASE_ADDR+0x00)   //  32bit gpio ptd data direction reg
#define GPIOD_OCR1	      (GPIOD_BASE_ADDR+0x04)   //  32bit gpio ptd output config 1 reg
#define GPIOD_OCR2	      (GPIOD_BASE_ADDR+0x08)   //  32bit gpio ptd output config 2 reg
#define GPIOD_ICONFA1	      (GPIOD_BASE_ADDR+0x0C)   //  32bit gpio ptd input config A1 reg
#define GPIOD_ICONFA2	      (GPIOD_BASE_ADDR+0x10)   //  32bit gpio ptd input config A2 reg
#define GPIOD_ICONFB1	      (GPIOD_BASE_ADDR+0x14)   //  32bit gpio ptd input config B1 reg
#define GPIOD_ICONFB2	      (GPIOD_BASE_ADDR+0x18)   //  32bit gpio ptd input config B2 reg
#define GPIOD_DR	      (GPIOD_BASE_ADDR+0x1C)   //  32bit gpio ptd data reg
#define GPIOD_GIUS	      (GPIOD_BASE_ADDR+0x20)   //  32bit gpio ptd in use reg
#define GPIOD_SSR	      (GPIOD_BASE_ADDR+0x24)   //  32bit gpio ptd sample status reg
#define GPIOD_ICR1	      (GPIOD_BASE_ADDR+0x28)   //  32bit gpio ptd interrupt ctrl 1 reg
#define GPIOD_ICR2	      (GPIOD_BASE_ADDR+0x2C)   //  32bit gpio ptd interrupt ctrl 2 reg
#define GPIOD_IMR	      (GPIOD_BASE_ADDR+0x30)   //  32bit gpio ptd interrupt mask reg
#define GPIOD_ISR	      (GPIOD_BASE_ADDR+0x34)   //  32bit gpio ptd interrupt status reg
#define GPIOD_GPR	      (GPIOD_BASE_ADDR+0x38)   //  32bit gpio ptd general purpose reg
#define GPIOD_SWR	      (GPIOD_BASE_ADDR+0x3C)   //  32bit gpio ptd software reset reg 
#define GPIOD_PUEN	      (GPIOD_BASE_ADDR+0x40)   //  32bit gpio ptd pull up enable reg
			
#define GPIOE_BASE_ADDR	       	0x10015400	
#define GPIOE_DDIR	      (GPIOE_BASE_ADDR+0x00)   //  32bit gpio pte data direction reg
#define GPIOE_OCR1	      (GPIOE_BASE_ADDR+0x04)   //  32bit gpio pte output config 1 reg
#define GPIOE_OCR2	      (GPIOE_BASE_ADDR+0x08)   //  32bit gpio pte output config 2 reg
#define GPIOE_ICONFA1	      (GPIOE_BASE_ADDR+0x0C)   //  32bit gpio pte input config A1 reg
#define GPIOE_ICONFA2	      (GPIOE_BASE_ADDR+0x10)   //  32bit gpio pte input config A2 reg
#define GPIOE_ICONFB1	      (GPIOE_BASE_ADDR+0x14)   //  32bit gpio pte input config B1 reg
#define GPIOE_ICONFB2	      (GPIOE_BASE_ADDR+0x18)   //  32bit gpio pte input config B2 reg
#define GPIOE_DR	      (GPIOE_BASE_ADDR+0x1C)   //  32bit gpio pte data reg
#define GPIOE_GIUS	      (GPIOE_BASE_ADDR+0x20)   //  32bit gpio pte in use reg
#define GPIOE_SSR	      (GPIOE_BASE_ADDR+0x24)   //  32bit gpio pte sample status reg
#define GPIOE_ICR1	      (GPIOE_BASE_ADDR+0x28)   //  32bit gpio pte interrupt ctrl 1 reg
#define GPIOE_ICR2	      (GPIOE_BASE_ADDR+0x2C)   //  32bit gpio pte interrupt ctrl 2 reg
#define GPIOE_IMR	      (GPIOE_BASE_ADDR+0x30)   //  32bit gpio pte interrupt mask reg
#define GPIOE_ISR	      (GPIOE_BASE_ADDR+0x34)   //  32bit gpio pte interrupt status reg
#define GPIOE_GPR	      (GPIOE_BASE_ADDR+0x38)   //  32bit gpio pte general purpose reg
#define GPIOE_SWR	      (GPIOE_BASE_ADDR+0x3C)   //  32bit gpio pte software reset reg 
#define GPIOE_PUEN	      (GPIOE_BASE_ADDR+0x40)   //  32bit gpio pte pull up enable reg
			
#define GPIOF_BASE_ADDR	       	0x10015500	
#define GPIOF_DDIR	      (GPIOF_BASE_ADDR+0x00)   //  32bit gpio ptf data direction reg
#define GPIOF_OCR1	      (GPIOF_BASE_ADDR+0x04)   //  32bit gpio ptf output config 1 reg
#define GPIOF_OCR2	      (GPIOF_BASE_ADDR+0x08)   //  32bit gpio ptf output config 2 reg
#define GPIOF_ICONFA1	      (GPIOF_BASE_ADDR+0x0C)   //  32bit gpio ptf input config A1 reg
#define GPIOF_ICONFA2	      (GPIOF_BASE_ADDR+0x10)   //  32bit gpio ptf input config A2 reg
#define GPIOF_ICONFB1	      (GPIOF_BASE_ADDR+0x14)   //  32bit gpio ptf input config B1 reg
#define GPIOF_ICONFB2	      (GPIOF_BASE_ADDR+0x18)   //  32bit gpio ptf input config B2 reg
#define GPIOF_DR	      (GPIOF_BASE_ADDR+0x1C)   //  32bit gpio ptf data reg
#define GPIOF_GIUS	      (GPIOF_BASE_ADDR+0x20)   //  32bit gpio ptf in use reg
#define GPIOF_SSR	      (GPIOF_BASE_ADDR+0x24)   //  32bit gpio ptf sample status reg
#define GPIOF_ICR1	      (GPIOF_BASE_ADDR+0x28)   //  32bit gpio ptf interrupt ctrl 1 reg
#define GPIOF_ICR2	      (GPIOF_BASE_ADDR+0x2C)   //  32bit gpio ptf interrupt ctrl 2 reg
#define GPIOF_IMR	      (GPIOF_BASE_ADDR+0x30)   //  32bit gpio ptf interrupt mask reg
#define GPIOF_ISR	      (GPIOF_BASE_ADDR+0x34)   //  32bit gpio ptf interrupt status reg
#define GPIOF_GPR	      (GPIOF_BASE_ADDR+0x38)   //  32bit gpio ptf general purpose reg
#define GPIOF_SWR	      (GPIOF_BASE_ADDR+0x3C)   //  32bit gpio ptf software reset reg 
#define GPIOF_PUEN	      (GPIOF_BASE_ADDR+0x40)   //  32bit gpio ptf pull up enable reg
			
#define GPIO_REG_BASE	       	0x10015600	
#define GPIO_PMASK	      (GPIO_REG_BASE+0x00)   //  32bit gpio interrupt mask reg


//;#########################################			
//;# AIPI2                                 #			
//;# $1002_0000 to $1002_0FFF              #			
//;#########################################			
#define AIPI2_BASE_ADDR	      	0x10020000	
#define AIPI2_PSR0	      	(AIPI2_BASE_ADDR+0x00)	//  32bit Peripheral Size Reg 0
#define AIPI2_PSR1	      	(AIPI2_BASE_ADDR+0x04)	//  32bit Peripheral Size Reg 1
#define AIPI2_PAR	      	(AIPI2_BASE_ADDR+0x08)	//  32bit Peripheral Access Reg



// ;#########################################			
// ;# USBOTG                                #			
// ;# $1002_4000 to $1002_5FFF              #			
// ;#########################################			
#define OTG_BASE_ADDR	       	0x10024000	
#define OTG_CORE_BASE	       	(OTG_BASE_ADDR+0x000)		//  base location for core
#define OTG_FUNC_BASE	       	(OTG_BASE_ADDR+0x040)		//  base location for function
#define OTG_HOST_BASE	       	(OTG_BASE_ADDR+0x080)		//  base location for host
#define OTG_DMA_BASE	       	(OTG_BASE_ADDR+0x800)		//  base location for dma
#define OTG_I2C_BASE		OTG_BASE_ADDR+0x100


			
#define OTG_ETD_BASE	       	(OTG_BASE_ADDR+0x200)		//  base location for etd memory
#define OTG_EP_BASE	       	(OTG_BASE_ADDR+0x400)		//  base location for ep memory
#define OTG_SYS_BASE	       	(OTG_BASE_ADDR+0x600)		//  base location for system
#define OTG_DATA_BASE	       	(OTG_BASE_ADDR+0x1000)	//  base location for data memory

#define OTG_SYS_CTRL	       		(OTG_SYS_BASE+0x000)	//  base location for system
			
#define OTG_CORE_HWMODE	       		(OTG_CORE_BASE+0x00)     //  32bit core hardware mode reg
#define OTG_CORE_CINT_STAT	       	(OTG_CORE_BASE+0x04)     //  32bit core int status reg
#define OTG_CORE_CINT_STEN	       	(OTG_CORE_BASE+0x08)     //  32bit core int enable reg
#define OTG_CORE_CLK_CTRL	       	(OTG_CORE_BASE+0x0C)     //  32bit core clock control reg
#define OTG_CORE_RST_CTRL	       	(OTG_CORE_BASE+0x10)     //  32bit core reset control reg
#define OTG_CORE_FRM_INTVL	       	(OTG_CORE_BASE+0x14)     //  32bit core frame interval reg
#define OTG_CORE_FRM_REMAIN	       	(OTG_CORE_BASE+0x18)     //  32bit core frame remaining reg
#define OTG_CORE_HNP_CSTAT	       	(OTG_CORE_BASE+0x1C)     //  32bit core HNP current state reg
#define OTG_CORE_HNP_TIMER1	       	(OTG_CORE_BASE+0x20)     //  32bit core HNP timer 1 reg
#define OTG_CORE_HNP_TIMER2	       	(OTG_CORE_BASE+0x24)     //  32bit core HNP timer 2 reg
#define OTG_CORE_HNP_T3PCR	       	(OTG_CORE_BASE+0x28)     //  32bit core HNP timer 3 pulse ctrl
#define OTG_CORE_HINT_STAT	       	(OTG_CORE_BASE+0x2C)     //  32bit core HNP int status reg
#define OTG_CORE_HINT_STEN	       	(OTG_CORE_BASE+0x30)     //  32bit core HNP int enable reg
			
#define OTG_FUNC_CND_STAT	       	(OTG_FUNC_BASE+0x00)     //  32bit func command status reg
#define OTG_FUNC_DEV_ADDR	       	(OTG_FUNC_BASE+0x04)     //  32bit func device address reg
#define OTG_FUNC_SINT_STAT	       	(OTG_FUNC_BASE+0x08)     //  32bit func system int status reg
#define OTG_FUNC_SINT_STEN	       	(OTG_FUNC_BASE+0x0C)     //  32bit func system int enable reg
#define OTG_FUNC_XINT_STAT	       	(OTG_FUNC_BASE+0x10)     //  32bit func X buf int status reg
#define OTG_FUNC_YINT_STAT	       	(OTG_FUNC_BASE+0x14)     //  32bit func Y buf int status reg
#define OTG_FUNC_XYINT_STEN	       	(OTG_FUNC_BASE+0x18)     //  32bit func XY buf int enable reg
#define OTG_FUNC_XFILL_STAT	       	(OTG_FUNC_BASE+0x1C)     //  32bit func X filled status reg
#define OTG_FUNC_YFILL_STAT	       	(OTG_FUNC_BASE+0x20)     //  32bit func Y filled status reg
#define OTG_FUNC_EP_EN	       		(OTG_FUNC_BASE+0x24)     //  32bit func endpoints enable reg
#define OTG_FUNC_EP_RDY	       		(OTG_FUNC_BASE+0x28)     //  32bit func endpoints ready reg
#define OTG_FUNC_IINT	       		(OTG_FUNC_BASE+0x2C)     //  32bit func immediate interrupt reg
#define OTG_FUNC_EP_DSTAT	       	(OTG_FUNC_BASE+0x30)     //  32bit func endpoints done status
#define OTG_FUNC_EP_DEN	       		(OTG_FUNC_BASE+0x34)     //  32bit func endpoints done enable
#define OTG_FUNC_EP_TOGGLE	       	(OTG_FUNC_BASE+0x38)     //  32bit func endpoints toggle bits
#define OTG_FUNC_FRM_NUM	       	(OTG_FUNC_BASE+0x3C)     //  32bit func frame number reg
			
#define OTG_HOST_CTRL	       		(OTG_HOST_BASE+0x00)     //  32bit host controller config reg
#define OTG_HOST_SINT_STAT	       	(OTG_HOST_BASE+0x08)     //  32bit host system int status reg
#define OTG_HOST_SINT_STEN	       	(OTG_HOST_BASE+0x0C)     //  32bit host system int enable reg
#define OTG_HOST_XINT_STAT	       	(OTG_HOST_BASE+0x18)     //  32bit host X buf int status reg
#define OTG_HOST_YINT_STAT	       	(OTG_HOST_BASE+0x1C)     //  32bit host Y buf int status reg
#define OTG_HOST_XYINT_STEN	       	(OTG_HOST_BASE+0x20)     //  32bit host XY buf int enable reg
#define OTG_HOST_XFILL_STAT	       	(OTG_HOST_BASE+0x28)     //  32bit host X filled status reg
#define OTG_HOST_YFILL_STAT	       	(OTG_HOST_BASE+0x2C)     //  32bit host Y filled status reg
#define OTG_HOST_ETD_EN	       		(OTG_HOST_BASE+0x40)     //  32bit host ETD enables reg
#define OTG_HOST_DIR_ROUTE	       	(OTG_HOST_BASE+0x48)     //  32bit host direct routing reg
#define OTG_HOST_IINT	       		(OTG_HOST_BASE+0x4C)     //  32bit host immediate interrupt reg
#define OTG_HOST_EP_DSTAT	       	(OTG_HOST_BASE+0x50)     //  32bit host endpoints done status
#define OTG_HOST_ETD_DONE	       	(OTG_HOST_BASE+0x54)     //  32bit host ETD done reg
#define OTG_HOST_FRM_NUM	       	(OTG_HOST_BASE+0x60)     //  32bit host frame number reg
#define OTG_HOST_LSP_THRESH	       	(OTG_HOST_BASE+0x64)     //  32bit host low speed threshold reg
#define OTG_HOST_HUB_DESCA	       	(OTG_HOST_BASE+0x68)     //  32bit host root hub descriptor A
#define OTG_HOST_HUB_DESCB	       	(OTG_HOST_BASE+0x6C)     //  32bit host root hub descriptor B
#define OTG_HOST_HUB_STAT	       	(OTG_HOST_BASE+0x70)     //  32bit host root hub status reg
#define OTG_HOST_PORT1_STAT	       	(OTG_HOST_BASE+0x74)     //  32bit host port 1 status bits
#define OTG_HOST_PORT2_STAT	       	(OTG_HOST_BASE+0x78)     //  32bit host port 2 status bits
#define OTG_HOST_PORT3_STAT	       	(OTG_HOST_BASE+0x7c)     //  32bit host port 3 status bits
			
#define OTG_DMA_REV_NUM	       		(OTG_DMA_BASE+0x000)     //  32bit dma revision number reg
#define OTG_DMA_DINT_STAT	       	(OTG_DMA_BASE+0x004)     //  32bit dma int status reg
#define OTG_DMA_DINT_STEN	       	(OTG_DMA_BASE+0x008)     //  32bit dma int enable reg
#define OTG_DMA_ETD_ERR	       		(OTG_DMA_BASE+0x00C)     //  32bit dma ETD error status reg
#define OTG_DMA_EP_ERR	       		(OTG_DMA_BASE+0x010)     //  32bit dma EP error status reg
#define OTG_DMA_ETD_EN	       		(OTG_DMA_BASE+0x020)     //  32bit dma ETD DMA enable reg
#define OTG_DMA_EP_EN	       		(OTG_DMA_BASE+0x024)     //  32bit dma EP DMA enable reg
#define OTG_DMA_ETD_ENXREQ	       	(OTG_DMA_BASE+0x028)     //  32bit dma ETD DMA enable Xtrig req
#define OTG_DMA_EP_ENXREQ	       	(OTG_DMA_BASE+0x02C)     //  32bit dma EP DMA enable Ytrig req
#define OTG_DMA_ETD_ENXYREQ	       	(OTG_DMA_BASE+0x030)     //  32bit dma ETD DMA enble XYtrig req
#define OTG_DMA_EP_ENXYREQ	       	(OTG_DMA_BASE+0x034)     //  32bit dma EP DMA enable XYtrig req
#define OTG_DMA_ETD_BURST4	       	(OTG_DMA_BASE+0x038)     //  32bit dma ETD DMA enble burst4 reg
#define OTG_DMA_EP_BURST4	       	(OTG_DMA_BASE+0x03C)     //  32bit dma EP DMA enable burst4 reg
#define OTG_DMA_MISC_CTRL	       	(OTG_DMA_BASE+0x040)     //  32bit dma EP misc control reg
#define OTG_DMA_ETD_CH_CLR	       	(OTG_DMA_BASE+0x044)     //  32bit dma ETD clear channel reg
#define OTG_DMA_EP_CH_CLR	       	(OTG_DMA_BASE+0x048)     //  32bit dma EP clear channel reg
			
#define OTG_DMA_ETD0_MSA	       	(OTG_DMA_BASE+0x100)     //  32bit dma ETD0 mem start addr reg
#define OTG_DMA_ETD1_MSA	       	(OTG_DMA_BASE+0x104)     //  32bit dma ETD1 mem start addr reg
#define OTG_DMA_ETD2_MSA	       	(OTG_DMA_BASE+0x108)     //  32bit dma ETD2 mem start addr reg
#define OTG_DMA_ETD3_MSA	       	(OTG_DMA_BASE+0x10C)     //  32bit dma ETD3 mem start addr reg
#define OTG_DMA_ETD4_MSA	       	(OTG_DMA_BASE+0x110)     //  32bit dma ETD4 mem start addr reg
#define OTG_DMA_ETD5_MSA	       	(OTG_DMA_BASE+0x114)     //  32bit dma ETD5 mem start addr reg
#define OTG_DMA_ETD6_MSA	       	(OTG_DMA_BASE+0x118)     //  32bit dma ETD6 mem start addr reg
#define OTG_DMA_ETD7_MSA	       	(OTG_DMA_BASE+0x11C)     //  32bit dma ETD7 mem start addr reg
#define OTG_DMA_ETD8_MSA	       	(OTG_DMA_BASE+0x120)     //  32bit dma ETD8 mem start addr reg
#define OTG_DMA_ETD9_MSA	       	(OTG_DMA_BASE+0x124)     //  32bit dma ETD9 mem start addr reg
#define OTG_DMA_ETD10_MSA	       	(OTG_DMA_BASE+0x128)     //  32bit dma ETD10 mem start addr reg
#define OTG_DMA_ETD11_MSA	       	(OTG_DMA_BASE+0x12C)     //  32bit dma ETD11 mem start addr reg
#define OTG_DMA_ETD12_MSA	       	(OTG_DMA_BASE+0x130)     //  32bit dma ETD12 mem start addr reg
#define OTG_DMA_ETD13_MSA	       	(OTG_DMA_BASE+0x134)     //  32bit dma ETD13 mem start addr reg
#define OTG_DMA_ETD14_MSA	       	(OTG_DMA_BASE+0x138)     //  32bit dma ETD14 mem start addr reg
#define OTG_DMA_ETD15_MSA	       	(OTG_DMA_BASE+0x13C)     //  32bit dma ETD15 mem start addr reg
#define OTG_DMA_ETD16_MSA	       	(OTG_DMA_BASE+0x140)     //  32bit dma ETD16 mem start addr reg
#define OTG_DMA_ETD17_MSA	       	(OTG_DMA_BASE+0x144)     //  32bit dma ETD17 mem start addr reg
#define OTG_DMA_ETD18_MSA	       	(OTG_DMA_BASE+0x148)     //  32bit dma ETD18 mem start addr reg
#define OTG_DMA_ETD19_MSA	       	(OTG_DMA_BASE+0x14C)     //  32bit dma ETD19 mem start addr reg
#define OTG_DMA_ETD20_MSA	       	(OTG_DMA_BASE+0x150)     //  32bit dma ETD20 mem start addr reg
#define OTG_DMA_ETD21_MSA	       	(OTG_DMA_BASE+0x154)     //  32bit dma ETD21 mem start addr reg
#define OTG_DMA_ETD22_MSA	       	(OTG_DMA_BASE+0x158)     //  32bit dma ETD22 mem start addr reg
#define OTG_DMA_ETD23_MSA	       	(OTG_DMA_BASE+0x15C)     //  32bit dma ETD23 mem start addr reg
#define OTG_DMA_ETD24_MSA	       	(OTG_DMA_BASE+0x160)     //  32bit dma ETD24 mem start addr reg
#define OTG_DMA_ETD25_MSA	       	(OTG_DMA_BASE+0x164)     //  32bit dma ETD25 mem start addr reg
#define OTG_DMA_ETD26_MSA	       	(OTG_DMA_BASE+0x168)     //  32bit dma ETD26 mem start addr reg
#define OTG_DMA_ETD27_MSA	       	(OTG_DMA_BASE+0x16C)     //  32bit dma ETD27 mem start addr reg
#define OTG_DMA_ETD28_MSA	       	(OTG_DMA_BASE+0x170)     //  32bit dma ETD28 mem start addr reg
#define OTG_DMA_ETD29_MSA	       	(OTG_DMA_BASE+0x174)     //  32bit dma ETD29 mem start addr reg
#define OTG_DMA_ETD30_MSA	       	(OTG_DMA_BASE+0x178)     //  32bit dma ETD30 mem start addr reg
#define OTG_DMA_ETD31_MSA	       	(OTG_DMA_BASE+0x17C)     //  32bit dma ETD31 mem start addr reg
			
#define OTG_DMA_EP0_O_MSA	       	(OTG_DMA_BASE+0x180)     //  32bit dma EP0 o/p mem start addr
#define OTG_DMA_EP0_I_MSA	       	(OTG_DMA_BASE+0x184)     //  32bit dma EP0 i/p mem start addr
#define OTG_DMA_EP1_O_MSA	       	(OTG_DMA_BASE+0x188)     //  32bit dma EP1 o/p mem start addr
#define OTG_DMA_EP1_I_MSA	       	(OTG_DMA_BASE+0x18C)     //  32bit dma EP1 i/p mem start addr
#define OTG_DMA_EP2_O_MSA	       	(OTG_DMA_BASE+0x190)     //  32bit dma EP2 o/p mem start addr
#define OTG_DMA_EP2_I_MSA	       	(OTG_DMA_BASE+0x194)     //  32bit dma EP2 i/p mem start addr
#define OTG_DMA_EP3_O_MSA	       	(OTG_DMA_BASE+0x198)     //  32bit dma EP3 o/p mem start addr
#define OTG_DMA_EP3_I_MSA	       	(OTG_DMA_BASE+0x19C)     //  32bit dma EP3 i/p mem start addr
#define OTG_DMA_EP4_O_MSA	       	(OTG_DMA_BASE+0x1A0)     //  32bit dma EP4 o/p mem start addr
#define OTG_DMA_EP4_I_MSA	       	(OTG_DMA_BASE+0x1A4)     //  32bit dma EP4 i/p mem start addr
#define OTG_DMA_EP5_O_MSA	       	(OTG_DMA_BASE+0x1A8)     //  32bit dma EP5 o/p mem start addr
#define OTG_DMA_EP5_I_MSA	       	(OTG_DMA_BASE+0x1AC)     //  32bit dma EP5 i/p mem start addr
#define OTG_DMA_EP6_O_MSA	       	(OTG_DMA_BASE+0x1B0)     //  32bit dma EP6 o/p mem start addr
#define OTG_DMA_EP6_I_MSA	       	(OTG_DMA_BASE+0x1B4)     //  32bit dma EP6 i/p mem start addr
#define OTG_DMA_EP7_O_MSA	       	(OTG_DMA_BASE+0x1B8)     //  32bit dma EP7 o/p mem start addr
#define OTG_DMA_EP7_I_MSA	       	(OTG_DMA_BASE+0x1BC)     //  32bit dma EP7 i/p mem start addr
#define OTG_DMA_EP8_O_MSA	       	(OTG_DMA_BASE+0x1C0)     //  32bit dma EP8 o/p mem start addr
#define OTG_DMA_EP8_I_MSA	       	(OTG_DMA_BASE+0x1C4)     //  32bit dma EP8 i/p mem start addr
#define OTG_DMA_EP9_O_MSA	       	(OTG_DMA_BASE+0x1C8)     //  32bit dma EP9 o/p mem start addr
#define OTG_DMA_EP9_I_MSA	       	(OTG_DMA_BASE+0x1CC)     //  32bit dma EP9 i/p mem start addr
#define OTG_DMA_EP10_O_MSA	       	(OTG_DMA_BASE+0x1D0)     //  32bit dma EP10 o/p mem start addr
#define OTG_DMA_EP10_I_MSA	       	(OTG_DMA_BASE+0x1D4)     //  32bit dma EP10 i/p mem start addr
#define OTG_DMA_EP11_O_MSA	       	(OTG_DMA_BASE+0x1D8)     //  32bit dma EP11 o/p mem start addr
#define OTG_DMA_EP11_I_MSA	       	(OTG_DMA_BASE+0x1DC)     //  32bit dma EP11 i/p mem start addr
#define OTG_DMA_EP12_O_MSA	       	(OTG_DMA_BASE+0x1E0)     //  32bit dma EP12 o/p mem start addr
#define OTG_DMA_EP12_I_MSA	       	(OTG_DMA_BASE+0x1E4)     //  32bit dma EP12 i/p mem start addr
#define OTG_DMA_EP13_O_MSA	       	(OTG_DMA_BASE+0x1E8)     //  32bit dma EP13 o/p mem start addr
#define OTG_DMA_EP13_I_MSA	       	(OTG_DMA_BASE+0x1EC)     //  32bit dma EP13 i/p mem start addr
#define OTG_DMA_EP14_O_MSA	       	(OTG_DMA_BASE+0x1F0)     //  32bit dma EP14 o/p mem start addr
#define OTG_DMA_EP14_I_MSA	       	(OTG_DMA_BASE+0x1F4)     //  32bit dma EP14 i/p mem start addr
#define OTG_DMA_EP15_O_MSA	       	(OTG_DMA_BASE+0x1F8)     //  32bit dma EP15 o/p mem start addr
#define OTG_DMA_EP15_I_MSA	       	(OTG_DMA_BASE+0x1FC)     //  32bit dma EP15 i/p mem start addr
			
#define OTG_DMA_ETD0_BPTR	       	(OTG_DMA_BASE+0x200)     //  32bit dma ETD0 buf tx pointer reg
#define OTG_DMA_ETD1_BPTR	       	(OTG_DMA_BASE+0x204)     //  32bit dma ETD1 buf tx pointer reg
#define OTG_DMA_ETD2_BPTR	       	(OTG_DMA_BASE+0x208)     //  32bit dma ETD2 buf tx pointer reg
#define OTG_DMA_ETD3_BPTR	       	(OTG_DMA_BASE+0x20C)     //  32bit dma ETD3 buf tx pointer reg
#define OTG_DMA_ETD4_BPTR	       	(OTG_DMA_BASE+0x210)     //  32bit dma ETD4 buf tx pointer reg
#define OTG_DMA_ETD5_BPTR	       	(OTG_DMA_BASE+0x214)     //  32bit dma ETD5 buf tx pointer reg
#define OTG_DMA_ETD6_BPTR	       	(OTG_DMA_BASE+0x218)     //  32bit dma ETD6 buf tx pointer reg
#define OTG_DMA_ETD7_BPTR	       	(OTG_DMA_BASE+0x21C)     //  32bit dma ETD7 buf tx pointer reg
#define OTG_DMA_ETD8_BPTR	       	(OTG_DMA_BASE+0x220)     //  32bit dma ETD8 buf tx pointer reg
#define OTG_DMA_ETD9_BPTR	       	(OTG_DMA_BASE+0x224)     //  32bit dma ETD9 buf tx pointer reg
#define OTG_DMA_ETD10_BPTR	       	(OTG_DMA_BASE+0x228)     //  32bit dma ETD10 buf tx pointer reg
#define OTG_DMA_ETD11_BPTR	       	(OTG_DMA_BASE+0x22C)     //  32bit dma ETD11 buf tx pointer reg
#define OTG_DMA_ETD12_BPTR	       	(OTG_DMA_BASE+0x230)     //  32bit dma ETD12 buf tx pointer reg
#define OTG_DMA_ETD13_BPTR	       	(OTG_DMA_BASE+0x234)     //  32bit dma ETD13 buf tx pointer reg
#define OTG_DMA_ETD14_BPTR	       	(OTG_DMA_BASE+0x238)     //  32bit dma ETD14 buf tx pointer reg
#define OTG_DMA_ETD15_BPTR	       	(OTG_DMA_BASE+0x23C)     //  32bit dma ETD15 buf tx pointer reg
#define OTG_DMA_ETD16_BPTR	       	(OTG_DMA_BASE+0x240)     //  32bit dma ETD16 buf tx pointer reg
#define OTG_DMA_ETD17_BPTR	       	(OTG_DMA_BASE+0x244)     //  32bit dma ETD17 buf tx pointer reg
#define OTG_DMA_ETD18_BPTR	       	(OTG_DMA_BASE+0x248)     //  32bit dma ETD18 buf tx pointer reg
#define OTG_DMA_ETD19_BPTR	       	(OTG_DMA_BASE+0x24C)     //  32bit dma ETD19 buf tx pointer reg
#define OTG_DMA_ETD20_BPTR	       	(OTG_DMA_BASE+0x250)     //  32bit dma ETD20 buf tx pointer reg
#define OTG_DMA_ETD21_BPTR	       	(OTG_DMA_BASE+0x254)     //  32bit dma ETD21 buf tx pointer reg
#define OTG_DMA_ETD22_BPTR	       	(OTG_DMA_BASE+0x258)     //  32bit dma ETD22 buf tx pointer reg
#define OTG_DMA_ETD23_BPTR	       	(OTG_DMA_BASE+0x25C)     //  32bit dma ETD23 buf tx pointer reg
#define OTG_DMA_ETD24_BPTR	       	(OTG_DMA_BASE+0x260)     //  32bit dma ETD24 buf tx pointer reg
#define OTG_DMA_ETD25_BPTR	       	(OTG_DMA_BASE+0x264)     //  32bit dma ETD25 buf tx pointer reg
#define OTG_DMA_ETD26_BPTR	       	(OTG_DMA_BASE+0x268)     //  32bit dma ETD26 buf tx pointer reg
#define OTG_DMA_ETD27_BPTR	       	(OTG_DMA_BASE+0x26C)     //  32bit dma ETD27 buf tx pointer reg
#define OTG_DMA_ETD28_BPTR	       	(OTG_DMA_BASE+0x270)     //  32bit dma ETD28 buf tx pointer reg
#define OTG_DMA_ETD29_BPTR	       	(OTG_DMA_BASE+0x274)     //  32bit dma ETD29 buf tx pointer reg
#define OTG_DMA_ETD30_BPTR	       	(OTG_DMA_BASE+0x278)     //  32bit dma ETD30 buf tx pointer reg
#define OTG_DMA_ETD31_BPTR	       	(OTG_DMA_BASE+0x27C)     //  32bit dma ETD31 buf tx pointer reg
			
#define OTG_DMA_EP0_O_BPTR	       	(OTG_DMA_BASE+0x280)     //  32bit dma EP0 o/p buf tx pointer
#define OTG_DMA_EP0_I_BPTR	       	(OTG_DMA_BASE+0x284)     //  32bit dma EP0 i/p buf tx pointer
#define OTG_DMA_EP1_O_BPTR	       	(OTG_DMA_BASE+0x288)     //  32bit dma EP1 o/p buf tx pointer
#define OTG_DMA_EP1_I_BPTR	       	(OTG_DMA_BASE+0x28C)     //  32bit dma EP1 i/p buf tx pointer
#define OTG_DMA_EP2_O_BPTR	       	(OTG_DMA_BASE+0x290)     //  32bit dma EP2 o/p buf tx pointer
#define OTG_DMA_EP2_I_BPTR	       	(OTG_DMA_BASE+0x294)     //  32bit dma EP2 i/p buf tx pointer
#define OTG_DMA_EP3_O_BPTR	       	(OTG_DMA_BASE+0x298)     //  32bit dma EP3 o/p buf tx pointer
#define OTG_DMA_EP3_I_BPTR	       	(OTG_DMA_BASE+0x29C)     //  32bit dma EP3 i/p buf tx pointer
#define OTG_DMA_EP4_O_BPTR	       	(OTG_DMA_BASE+0x2A0)     //  32bit dma EP4 o/p buf tx pointer
#define OTG_DMA_EP4_I_BPTR	       	(OTG_DMA_BASE+0x2A4)     //  32bit dma EP4 i/p buf tx pointer
#define OTG_DMA_EP5_O_BPTR	       	(OTG_DMA_BASE+0x2A8)     //  32bit dma EP5 o/p buf tx pointer
#define OTG_DMA_EP5_I_BPTR	       	(OTG_DMA_BASE+0x2AC)     //  32bit dma EP5 i/p buf tx pointer
#define OTG_DMA_EP6_O_BPTR	       	(OTG_DMA_BASE+0x2B0)     //  32bit dma EP6 o/p buf tx pointer
#define OTG_DMA_EP6_I_BPTR	       	(OTG_DMA_BASE+0x2B4)     //  32bit dma EP6 i/p buf tx pointer
#define OTG_DMA_EP7_O_BPTR	       	(OTG_DMA_BASE+0x2B8)     //  32bit dma EP7 o/p buf tx pointer
#define OTG_DMA_EP7_I_BPTR	       	(OTG_DMA_BASE+0x2BC)     //  32bit dma EP7 i/p buf tx pointer
#define OTG_DMA_EP8_O_BPTR	       	(OTG_DMA_BASE+0x2C0)     //  32bit dma EP8 o/p buf tx pointer
#define OTG_DMA_EP8_I_BPTR	       	(OTG_DMA_BASE+0x2C4)     //  32bit dma EP8 i/p buf tx pointer
#define OTG_DMA_EP9_O_BPTR	       	(OTG_DMA_BASE+0x2C8)     //  32bit dma EP9 o/p buf tx pointer
#define OTG_DMA_EP9_I_BPTR	       	(OTG_DMA_BASE+0x2CC)     //  32bit dma EP9 i/p buf tx pointer
#define OTG_DMA_EP10_O_BPTR	       	(OTG_DMA_BASE+0x2D0)     //  32bit dma EP10 o/p buf tx pointer
#define OTG_DMA_EP10_I_BPTR	       	(OTG_DMA_BASE+0x2D4)     //  32bit dma EP10 i/p buf tx pointer
#define OTG_DMA_EP11_O_BPTR	       	(OTG_DMA_BASE+0x2D8)     //  32bit dma EP11 o/p buf tx pointer
#define OTG_DMA_EP11_I_BPTR	       	(OTG_DMA_BASE+0x2DC)     //  32bit dma EP11 i/p buf tx pointer
#define OTG_DMA_EP12_O_BPTR	       	(OTG_DMA_BASE+0x2E0)     //  32bit dma EP12 o/p buf tx pointer
#define OTG_DMA_EP12_I_BPTR	       	(OTG_DMA_BASE+0x2E4)     //  32bit dma EP12 i/p buf tx pointer
#define OTG_DMA_EP13_O_BPTR	       	(OTG_DMA_BASE+0x2E8)     //  32bit dma EP13 o/p buf tx pointer
#define OTG_DMA_EP13_I_BPTR	       	(OTG_DMA_BASE+0x2EC)     //  32bit dma EP13 i/p buf tx pointer
#define OTG_DMA_EP14_O_BPTR	       	(OTG_DMA_BASE+0x2F0)     //  32bit dma EP14 o/p buf tx pointer
#define OTG_DMA_EP14_I_BPTR	       	(OTG_DMA_BASE+0x2F4)     //  32bit dma EP14 i/p buf tx pointer
#define OTG_DMA_EP15_O_BPTR	       	(OTG_DMA_BASE+0x2F8)     //  32bit dma EP15 o/p buf tx pointer
#define OTG_DMA_EP15_I_BPTR	       	(OTG_DMA_BASE+0x2FC)     //  32bit dma EP15 i/p buf tx pointer

#define VENDOR_ID_REG0_ADD      	OTG_I2C_BASE+0x00
#define VENDOR_ID_REG1_ADD      	OTG_I2C_BASE+0x01
#define PRODUCT_ID_REG0_ADD     	OTG_I2C_BASE+0x02
#define PRODUCT_ID_REG1_ADD     	OTG_I2C_BASE+0x03
#define MODE_REG1_SET_ADD       	OTG_I2C_BASE+0x04
#define MODE_REG1_CLR_ADD       	OTG_I2C_BASE+0x05
#define OTG_CTRL_REG1_SET_ADD   	OTG_I2C_BASE+0x06
#define OTG_CTRL_REG1_CLR_ADD  		OTG_I2C_BASE+0x07
#define INT_SRC_REG_ADD        	 	OTG_I2C_BASE+0x08
#define INT_LAT_REG_SET_ADD     	OTG_I2C_BASE+0x0a
#define INT_LAT_REG_CLR_ADD     	OTG_I2C_BASE+0x0b
#define INT_FALSE_REG_SET_ADD   	OTG_I2C_BASE+0x0c
#define INT_FALSE_REG_CLR_ADD    	OTG_I2C_BASE+0x0d
#define INT_TRUE_REG_SET_ADD     	OTG_I2C_BASE+0x0e
#define INT_TRUE_REG_CLR_ADD     	OTG_I2C_BASE+0x0f
#define OTG_CTRL_REG2_SET_ADD    	OTG_I2C_BASE+0x10
#define OTG_CTRL_REG2_CLR_ADD    	OTG_I2C_BASE+0x11
#define MODE_REG2_SET_ADD       	OTG_I2C_BASE+0x12
#define MODE_REG2_CLR_ADD       	OTG_I2C_BASE+0x13
#define BCD_DEV_REG0_ADD        	OTG_I2C_BASE+0x14
#define BCD_DEV_REG1_ADD        	OTG_I2C_BASE+0x15

#define OTG_XCVR_DEVAD_ADD      	OTG_I2C_BASE+0x18
#define SEQ_OP_REG_ADD         		OTG_I2C_BASE+0x19
#define SEQ_RD_STARTAD_ADD      	OTG_I2C_BASE+0x1a
#define I2C_OP_CTRL_REG_ADD      	OTG_I2C_BASE+0x1b
#define SCLK_TO_SCL_HPER_ADD     	OTG_I2C_BASE+0x1e
#define I2C_MASTER_INT_REG_ADD   	OTG_I2C_BASE+0x1f

//
// ;---------------------------------------;
// ; Clock & Reset (CRM)                   ;
// ; &1002_7000 to $1002_7020              ;
// ;---------------------------------------;
#define CRM_BASE_ADDR           0x10027000                
#define CRM_CSCR                CRM_BASE_ADDR           // ; Clock Source Control Reg
#define CRM_MPCTL0              (CRM_BASE_ADDR+0x04)    // ; MCU PLL Control Reg      
#define CRM_MPCTL1              (CRM_BASE_ADDR+0x08)  // ; MCU PLL & System Clk Ctl Reg
#define CRM_SPCTL0              (CRM_BASE_ADDR+0x0C)    // ; USB PLL Control Reg 0
#define CRM_SPCTL1              (CRM_BASE_ADDR+0x10)    // ; USB PLL Control Reg 1
#define CRM_OSC26MCTL			(CRM_BASE_ADDR+0x14)    // ; Oscillator 26M Register
#define CRM_PCDR                (CRM_BASE_ADDR+0x18)  	// ; Perpheral Clock Divider Reg
#define CRM_PCCR0				(CRM_BASE_ADDR+0x1C)    // ; Perpheral Clock Control Reg0
#define CRM_PCCR1 				(CRM_BASE_ADDR+0x20)    // ; Perpheral Clock Control Reg1





//;#########################################			                                                                                                                                                                                                                                                                                    
//;# MAX                                   #			                                                                                                                                                                                                                                                                                    
//;# $1003_F000 to $1003_FFFF              #			                                                                                                                                                                                                                                                                                    
//;#########################################			                                                                                                                                                                                                                                                                                    
#define MAX_BASE_ADDR	    0x1003F000	                                                                                                                                                                                                                                                                                                    
#define MAX_SLV0_BASE	    	 	(MAX_BASE_ADDR+0x000)	//  base location for slave 0                                                                                                                                                                                                                                                                
#define MAX_SLV1_BASE	    		(MAX_BASE_ADDR+0x100)	//  base location for slave 1                                                                                                                                                                                                                                                                
#define MAX_SLV2_BASE	    		(MAX_BASE_ADDR+0x200)	//  base location for slave 2                                                                                                                                                                                                                                                                
#define MAX_SLV3_BASE	    		(MAX_BASE_ADDR+0x300)	// base location for slave 3                                                                                                                                                                                                                                                                
			    		                                                                                                                                                                                                                                                                                                    
#define MAX_SLV0_MPR0	    		(MAX_SLV0_BASE+0x00)	// 32bit max slv0 master priority reg                                                                                                                                                                                                                                                       
#define MAX_SLV0_AMPR0	    		(MAX_SLV0_BASE+0x04)	// 32bit max slv0 alt priority reg                                                                                                                                                                                                                                                          
#define MAX_SLV0_SGPCR0	    		(MAX_SLV0_BASE+0x10)	// 32bit max slv0 general ctrl reg                                                                                                                                                                                                                                                          
#define MAX_SLV0_ASGPCR0	    	(MAX_SLV0_BASE+0x14)	// 32bit max slv0 alt generl ctrl reg                                                                                                                                                                                                                                               
			                                                                                                                                                                                                                                                                                                                            
#define MAX_SLV1_MPR1	    		(MAX_SLV1_BASE+0x00)	// 32bit max slv1 master priority reg                                                                                                                                                                                                                                                       
#define MAX_SLV1_AMPR1	    		(MAX_SLV1_BASE+0x04)	// 32bit max slv1 alt priority reg                                                                                                                                                                                                                                                          
#define MAX_SLV1_SGPCR1	    		(MAX_SLV1_BASE+0x10)	// 32bit max slv1 general ctrl reg                                                                                                                                                                                                                                                          
#define MAX_SLV1_ASGPCR1	    	(MAX_SLV1_BASE+0x14)	// 32bit max slv1 alt generl ctrl reg                                                                                                                                                                                                                                               
			                                                                                                                                                                                                                                                                                                                            
#define MAX_SLV2_MPR2	    		(MAX_SLV2_BASE+0x00)	// 32bit max slv2 master priority reg                                                                                                                                                                                                                                                       
#define MAX_SLV2_AMPR2	    		(MAX_SLV2_BASE+0x04)	// 32bit max slv2 alt priority reg                                                                                                                                                                                                                                                          
#define MAX_SLV2_SGPCR2	    		(MAX_SLV2_BASE+0x10)	// 32bit max slv2 general ctrl reg                                                                                                                                                                                                                                                          
#define MAX_SLV2_ASGPCR2	    	(MAX_SLV2_BASE+0x14)	// 32bit max slv2 alt generl ctrl reg                                                                                                                                                                                                                                               
			                                                                                                                                                                                                                                                                                                                            
#define MAX_SLV3_MPR3	    		(MAX_SLV3_BASE+0x00)	// 32bit max slv3 master priority reg                                                                                                                                                                                                                                                       
#define MAX_SLV3_AMPR3	    		(MAX_SLV3_BASE+0x04)	// 32bit max slv3 alt priority reg                                                                                                                                                                                                                                                          
#define MAX_SLV3_SGPCR3	    		(MAX_SLV3_BASE+0x10)	// 32bit max slv3 general ctrl reg                                                                                                                                                                                                                                                          
#define MAX_SLV3_ASGPCR3	    	(MAX_SLV3_BASE+0x14)	// 32bit max slv3 alt generl ctrl reg                                                                                                                                                                                                                                               
			                                                                                                                                                                                                                                                                                                                            
#define MAX_MST0_MGPCR0	    		(MAX_BASE_ADDR+0x800)	// 32bit max mst0 general ctrl reg                                                                                                                                                                                                                                                          
#define MAX_MST1_MGPCR1	    		(MAX_BASE_ADDR+0x900)	// 32bit max mst1 general ctrl reg                                                                                                                                                                                                                                                          
#define MAX_MST2_MGPCR2	    		(MAX_BASE_ADDR+0xA00)	// 32bit max mst2 general ctrl reg                                                                                                                                                                                                                                                          
#define MAX_MST3_MGPCR3	    		(MAX_BASE_ADDR+0xB00)	// 32bit max mst3 general ctrl reg                                                                                                                                                                                                                                                          
#define MAX_MST4_MGPCR4	    		(MAX_BASE_ADDR+0xC00)	// 32bit max mst4 general ctrl reg                                                                                                                                                                                                                                                          
#define MAX_MST5_MGPCR5	    		(MAX_BASE_ADDR+0xD00)	// 32bit max mst5 general ctrl reg                                                                                                                                                                                                                                                          
			                                                                                                                                                                                                                                                                                                                            


//
// ;---------------------------------------;
// ; WEIM                                  ;
// ; $DF00_1000 to $DF00_1030              ;
// ;---------------------------------------;
#define WEIM_BASE_ADDR           0xDF001000                
#define WEIM_CS0U                EIM_BASE_ADDR                
#define WEIM_CS0L                (EIM_BASE_ADDR+0x04)                
#define WEIM_CS1U                (EIM_BASE_ADDR+0x08)                
#define WEIM_CS1L                (EIM_BASE_ADDR+0x0C)                
#define WEIM_CS2U                (EIM_BASE_ADDR+0x10)                
#define WEIM_CS2L                (EIM_BASE_ADDR+0x14)                
#define WEIM_CS3U                (EIM_BASE_ADDR+0x18)                
#define WEIM_CS3L                (EIM_BASE_ADDR+0x1C)                
#define WEIM_CS4U                (EIM_BASE_ADDR+0x20)                
#define WEIM_CS4L                (EIM_BASE_ADDR+0x24)                
#define WEIM_CS5U                (EIM_BASE_ADDR+0x28)                
#define WEIM_CS5L                (EIM_BASE_ADDR+0x2C)                
#define WEIM_EIM                 (EIM_BASE_ADDR+0x30)


#ifdef FPGA

#define EUART_BASE_ADDR		0x17000000
#define EUART_DR			(EUART_BASE_ADDR+0x0)           
#define EUART_RSR			(EUART_BASE_ADDR+0x4)           
#define EUART_ECR			(EUART_BASE_ADDR+0x4)           
#define EUART_LCRH			(EUART_BASE_ADDR+0x8)           
#define EUART_LCRM			(EUART_BASE_ADDR+0xC)           
#define EUART_LCRL			(EUART_BASE_ADDR+0x10)    
#define EUART_CR			(EUART_BASE_ADDR+0x14)           
#define EUART_FR			(EUART_BASE_ADDR+0x18)  
#define EUART_IIR			(EUART_BASE_ADDR+0x1C)           
#define EUARTI_ICR			(EUART_BASE_ADDR+0x1C)   

#define LM_AIPI  0xC0000000
#define LM_SW	 0xC0001014 	
//#define LM_SW	 0xC0000014 	


#define LED_BASE		0x1A000000
#define	LED_ALPHA		(LED_BASE+0x0)
#define LED_LIGHTS		(LED_BASE+0x4)
#define LED_SWITCHES	(LED_BASE+0x8)

#endif
/***************** End of Tahiti definition **************************/




/***************** UsbOtg definition **************************/

//EPConfig for EP0 OUT
#define EP0OutDW0	0x00080000	// configure EP0 for CTRL setup
#define EP0OutDW1	0x00000000	// XBuff and YBuff are 0
//#define EP0OutDW2				// Reserved
#define EP0OutDW3_8Byte	0x00E00008	// TotalByteCnt = 8 bytes
#define EP0OutDW3_Empty	0x00E00000	// TotalByteCnt = 0 Bytes


//EPConfig for EP0 IN (Empty Packet)
#define EP0InDW0	0x00080000	// configure EP0 for CTRL setup
#define EP0InDW1	0x00000020	// XBuff and YBuff are 0
//#define EP0InDW2				// Reserved
#define EP0InDW3_EMPTY	0x00E00000	// TotalByteCnt = 0 bytes

#define EP1OutDataMemoryOffset 0x20
#define EP1InDataMemoryOffset  0x30

#define CommandLength 	 0x10
#define AckCommandLength 	 0x4

/*otg control1
*/
#define I2C_MASK_DP_PULLUP      0x01 
#define I2C_MASK_DM_PULLUP      0x02  
#define I2C_MASK_DP_PULLDOWN    0x04 
#define I2C_MASK_DM_PULLDOWN    0x08 



#define EP_OUT_DIR       0
#define EP_IN_DIR        1
#define EP_NO_STALL      0
#define EP_STALL         1
#define EP_NO_SETUP      0
#define EP_SETUP         1

#define CTL_TYPE         0x0
#define BLK_TYPE         0x2


U32 EP_table_entry[32];
U32 EP_totalbyte[32];
U32 EP_active;			/*initialise all eps to be inactive */
//U32 dgcEpNextBufToService;	/*initialise to X buffers for all 32 entries.*/
U8 dgcEpFillDrainN;
U8 dgcEpXStatus;
U8 dgcEpYStatus;
U32 dgcEpStartAddress;
U32 EP_DMA_smsa[32];


U8 r_i2cDevAddr;   /* 1 byte */
U8 r_i2cRdData;    /* 1 byte */
U32 r_i2cHwSw;
U32  r_i2cStartAddr;


U8 rExpRegAddr; /* for saving the current start address for auto-checking */


/*============Function related tasks==========*/

void EPConfig(U32 ep, U32 dir, U32 stall, U32 setup, U32 MPS, U32 format, U32 xbsa, U32 ybsa, U32 Bufsize, U32 Tbyte);
void WriteEPToggleBit(U8 ep, U8 dir, U8 toggle_val);
void Enable_EP(U8 ep_num);
void Disable_EP(U8 ep_num);
U8 check_ep_ready(U8 qEpNumberDirection);
void toggle_ready_ep(U8 ep_num, U8 ready);
void Ready_EP_IN(U8 ep);
void Ready_EP_OUT(U8 ep);
void unready_ep_in(U8 ep);
void unready_ep_out(U8 ep);
void Disable_EP_OUT(U8 ep);
void Disable_EP_IN(U8 ep);
void WaitOnEPDone_OUT(U8 ep);
void Clear_active_ep(U8 ep_num);

/*=====Common tasks ======*/
void SetOtgPort_FunctionMode(void);
void OtgPort_ExtPullUpDisabled(void);
void OtgPort_PullUpEnabled(void);

/*=====i2c tasks ======= */
void i2cDpPullUp(void);

/* ------ newly added routine ------ */

void i2cSetSeqOpReg(U8 qWrData);
void i2cCheckBusy(void);
void i2cSetSeqRdStartAd(U8 qWrData);
void i2cSingleRegWrite(U32 qRegAddr, U8 qWrData);
void ToggleNextBufferToService(U8 mask);
void ClearXYBufferInt(void);
void EP0OutForRequest(void);
/***************** End of UsbOtg definition **************************/

